import QtQuick 1.1
import se.flir 1.0

// Box
// ---
/// The measurement box.

Rectangle {

    // Properties
    id: boxId
    x: toolX
    y: toolY
    width: boxWidth
    height: boxHeight
    color: Qt.rgba(0, 0, 0, 0)

    property bool moved: false
    property int dragStartWidth: 0
    property int dragStartHeight: 0
    property bool mouseEnabled: true

    // Debug output
    //Component.onCompleted: console.log("QML Box created")
    //Component.onDestruction: console.log("QML Box destroyed")

    // Properties needed if binding is broken by joystick navigation
    property int myX: toolX
    property int myY: toolY
    property int myWidth: boxWidth
    property int myHeight: boxHeight
    onMyXChanged:  {
        if (keyFilterTimer.running === false)
            x = myX
    }
    onMyYChanged:  {
        if (keyFilterTimer.running === false)
            y = myY
    }
    onMyWidthChanged:  {
        if (keyFilterTimer.running === false)
            width = myWidth
    }
    onMyHeightChanged:  {
        if (keyFilterTimer.running === false)
            height = myHeight
    }

    Timer {
        id: keyFilterTimer
        interval: 2000
        onTriggered:
        {
            x = toolX
            y = toolY
            width = boxWidth
            height = boxHeight
        }
    }

    property bool mySelection: selected
    onMySelectionChanged: {
        if (selected === true && activeFocus === false)
        {
            forceActiveFocus()
            keepFocusTimer.start()
            measureTools.sendEnterMFuncEditRequest(type, resIndex)
        }
        else if (selected === false) {
            menus.closeMenu()
        }
    }

    Timer {
        id: keepFocusTimer
        interval: 300
        onTriggered: if (selected === true && activeFocus === false) forceActiveFocus()
    }

    // Box
    Rectangle {
        id: selectedBoxId
        visible: selected
        anchors.fill: boxId
        border.color: "white"
        border.width: 1
        color: Qt.rgba(0, 0, 0, 0)
    }

    // Grab boxes
    Repeater {
        model: 5
        delegate: MeasureToolGrabBox {}
    }

    // Cold spot
    Image {
        x: minX - toolX - width / 2
        y: minY - toolY - height / 2
        source: "../images/Sc_MeasureToolPlaced_BoxSpotCold_Def.png"
        visible: minActive && x >= -width/2 && x < boxId.width-width/2 && y >= -height/2 && y < boxId.height-height/2
    }

    // Hot spot
    Image {
        x: maxX - toolX - width / 2
        y: maxY - toolY - height / 2
        source: "../images/Sc_MeasureToolPlaced_BoxSpotHot_Def.png"
        visible: maxActive && x >= -width/2 && x < boxId.width-width/2 && y >= -height/2 && y < boxId.height-height/2
    }

    // The ID
    Rectangle {
        id: idRect
        color: colors.transparent
        anchors.right: parent.right
        anchors.rightMargin: grid.horizontalSpacing
        anchors.top: parent.top
        anchors.topMargin: 1//grid.verticalSpacing
        width: idLabel.width + grid.horizontalSpacing
        height: idLabel.height
        visible: measureFuncs.boxCount > 1 ? true : false

        RoundedRect {
            anchors.fill: parent
        }

        Text {
            id: idLabel
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            text: resIndex + 1
            color: colors.textNormal
        }
    }

    // Local paramater indicator
    Rectangle {
        id: paramsRect
        color: colors.transparent
        anchors.left: idRect.right
        anchors.leftMargin: 2 * grid.horizontalSpacing
        anchors.top: parent.top
        anchors.topMargin: 1
        width: paramsLabel.width + grid.horizontalSpacing
        height: paramsLabel.height
        visible: localParams

        RoundedRect {
            anchors.fill: parent
        }

        Text {
            id: paramsLabel
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            //: One letter abbreviation of 'parameters'. It is used as a flag indicating that a measurement tool has parameter settings that differs from default.
            //% "P"
            text: qsTrId("ID_MEASURE_LOCAL_PARAMS_P") + translation.update
            color: colors.textNormal
        }
    }

    // Key events
    Keys.onPressed: {
        if (selected)
        {
            if (event.key === keys.up) {
                event.accepted = true
                if (selectionMoveMode)
                    y = Math.max(y - 10, 0)
                else
                {
                    var increase = y < 5 ? Math.abs(y) : 5
                    if (y + height + increase > grid.height)
                        increase = grid.height - y - height
                    y = y - increase
                    height = height + increase * 2
                }
            }
            else if (event.key === keys.down) {
                event.accepted = true
                if (selectionMoveMode)
                    y = Math.min(y + 10, grid.height - height - 1)
                else
                {
                    var decrease = height > 20 ? 10 : height - 10
                    y = y + decrease / 2
                    height = height - decrease
                }
            }
            else if (event.key === keys.left) {
                event.accepted = true
                if (selectionMoveMode)
                    x = Math.max(x - 10, 0)
                else
                {
                    var decreaseL = width > 20 ? 10 : width - 10
                    x = x + decreaseL / 2
                    width = width - decreaseL
                }
            }
            else if (event.key === keys.right) {
                event.accepted = true
                if (selectionMoveMode)
                    x = Math.min(x + 10, grid.width - width - grid.irLeftMargin - grid.irRightMargin - 1)
                else
                {
                    var increaseL = x < 5 ? Math.abs(x) : 5
                    if (x + width + increaseL > grid.width - grid.irLeftMargin - grid.irRightMargin)
                        increaseL = grid.width - x - width - grid.irLeftMargin - grid.irRightMargin
                    x = x - increaseL
                    width = width + increaseL * 2
                }
            }
        }
    }

    Keys.onReleased: {
        if (selected)
        {
            if (event.key === keys.up || event.key === keys.down ||
                event.key === keys.left || event.key === keys.right)
            {
                event.accepted = true
                keyFilterTimer.restart()
                if (event.isAutoRepeat === false)
                    measureFuncs.setPosition(type, resIndex, x, y, width, height)
            }
        }
    }
}
