import QtQuick 1.1

/**
 * This is a glue page.
 * The bluetooth page is part of the settings framework, but didn't fully fit as such we created
 * a ConnectionDiscoveryPage.qml
 * Now we have a new settings framework we likely want to port the wifi UI over to it, but
 * in the mean time this page bridges the 3 technologies.
 */
Item {
    property variant model: null
    signal hideSignal
    id: pageRoot

    function show() {
        bluetoothPage.show()
    }
    onModelChanged: {
        if (model) {
            bluetoothPage.model = model.nextModelObject()
            bluetoothPage.currentIndex = model.nextModelObject().currentIndex
            bluetoothPage.name = qsTrId(model.title)
            bluetoothPage.previous = null
            bluetoothPage.basePage = false
            model.nextModelObject().isNowActive(true)
        }
    }

    SettingsPage {
        width: parent.width
        height: parent.height
        id: bluetoothPage
        onHideSignal: parent.hideSignal()

        internalDelegate: WirelessSettingsDelegate {
            onClicked: {
                bluetoothPage.invokeCurrent("")
            }
        }

        property bool displayMessage: model === undefined ? false : model.displayMessage
        onDisplayMessageChanged: messageLoader.source = displayMessage ? "MessageBox.qml" : ""

        property bool displayPairingMessage: model === undefined
                                             ? false
                                             : model.displayPairingMessage
        onDisplayPairingMessageChanged: pairingMessageLoader.source = displayPairingMessage
                                        ? "MessageBox.qml"
                                        : ""

        // Dialog pop-up which can be shown if i.e. no devices are found.
        Loader {
            id: messageLoader
            onLoaded: {
                item.title = qsTrId("ID_SETUP_BLUETOOTH_NO_DEVICE_FOUND_TITLE") + translation.update
                item.text = qsTrId("ID_SETUP_BLUETOOTH_NO_DEVICE_FOUND_MESSAGE") + translation.update
                item.cancelButtontext = qsTrId("ID_BUTTON_CANCEL") + translation.update
                item.okButtontext = qsTrId("ID_SETUP_BLUETOOTH_NO_DEVICE_FOUND_REFRESH_BUTTON") + translation.update
                item.iconStyle = item.iconEnum.info

                item.show()
                item.forceActiveFocus()
            }
        }

        // Dialog pop-up which can be shown if i.e. no devices are found.
        Loader {
            id: pairingMessageLoader
            onLoaded: {
                item.title = qsTrId("ID_SETUP_BLUETOOTH_UNABLE_TO_PAIR_TITLE") + translation.update
                item.text = qsTrId("ID_SETUP_BLUETOOTH_UNABLE_TO_PAIR_BODY") + translation.update
                item.hasButtons = false
                item.iconStyle = item.iconEnum.info

                item.show()
                item.forceActiveFocus()
            }
        }

        Connections {
            target: messageLoader.item

            onCancelSelected: {
                bluetoothPage.model.displayMessage = false
                pageRoot.hideSignal()
            }

            onOkSelected: {
                bluetoothPage.model.displayMessage = false
                messageLoader.source = ""
                bluetoothPage.model.isNowActive(true)
            }
        }

        Connections {
            target: pairingMessageLoader.item
            onCancelSelected: bluetoothPage.model.displayPairingMessage = false
        }
    }
}
