import QtQuick 1.1

import se.flir 1.0
import System 1.0


// Desktop
// -------
// The "transparent" item that is the parent of all other QML elements.

FocusScope {

    // Signals

    signal showArchive
    signal triggerActionRequest
    signal exitEdit()
    property variant lastMenu: 0
    property bool cancellingPreview: false
    property bool cancellingPrompt: false

    // slots (called from the xml file for the globalMessageSpinner)
    function focusDesktopFocusGroup() {
//        console.log("focusDesktopFocusGroup() called")
        focus = true;
    }
    function cancelEdit() {
        cancellingPreview = false
        cancellingPrompt = false
        if (editHandler.isModified()) {
            editCancelDiag.showDialog()
        }
        else
        {
            editHandler.cancelEdit()
        }
    }
    function cancelPreview() {
        cancellingPrompt = false
        if (previewHandler.isModified()) {
            cancellingPreview = true
            editCancelDiag.showDialog()
        }
        else
        {
            previewHandler.cancelPreview()
        }
    }
    function cancelPrompt() {
        cancellingPreview = false
        if (promptHandler.isModified()) {
            cancellingPrompt = true
            editCancelDiag.showDialog()
        }
        else
        {
            promptHandler.cancelPrompt()
        }
    }

    function showTimeLapseDoneDialog() {
        timeLapseDoneDialog.showDialog()
    }


//    onFocusChanged: console.log("Desktop focus: " + focus)

    //todo: review if this is the right way to dispatch arrow keys
    signal key_down(int key, int repeat)
    signal key_up(int key, int repeat)

    // Property values

    id: desktop
    anchors.fill: parent
    width: grid.width
    height: grid.height

    property bool longTapped: false

    // Debug output

    onShowArchive: console.log("Desktop::showArchive()")

    // Touch and keyboard handling
    MouseArea {
        id: desktopMouseArea
        objectName: "desktopMouseArea"

        property int startx: 0
        property int starty: 0
        property bool dragged: false
        property bool startDrag: false
        anchors.fill: parent
        onPressed: {
            if (pip.selected)
                return
            else if (!menus.menuOpen) {
                longTapTimer.start()
                startx = mouse.x
                starty = mouse.y
//                console.log("Start move at " + startx + ", " + starty)
                dragged = false
                startDrag = true
            }
        }

        onMouseXChanged: {
            if (!startDrag)
                return
            if (!dragged && Math.abs(startx - mouse.x) > 5) {
                dragged = true
            } else if (dragged && greenbox.appState === GreenBox.FacetEditView) {
                var deltaX = (mouse.x - startx)
                var deltaY = (mouse.y - starty)
                if (deltaX != 0 || deltaY != 0) {
//                    console.log("move: " + deltaX + ", " + deltaY)
                    greenbox.system.setPanRel(-deltaX, -deltaY)
                    startx = mouse.x
                    starty = mouse.y
                }
            }

        }

        onMouseYChanged: {
            if (!startDrag)
                return
            if (!dragged && Math.abs(starty - mouse.y) > 5) {
                dragged = true
            } else if (dragged && greenbox.appState === GreenBox.FacetEditView) {
                var deltaX = (mouse.x - startx)
                var deltaY = (mouse.y - starty)
                if (deltaX != 0 || deltaY != 0) {
//                    console.log("move: " + deltaX + ", " + deltaY)
                    greenbox.system.setPanRel(-deltaX, -deltaY)
                    startx = mouse.x
                    starty = mouse.y
                }
            }
        }

        onReleased: {
            if (!dragged || menus.menuOpen)
            {
                // this is slighly tricky, the longTapTimer opens the menu, quickly followed by
                // the onReleased being called when the user releases the mouse. We have to avoid
                // closing it immediately after opening.
                if (menus.activeMenu === Menus.ZoomPopupMenu)
                {
                    // but later, if the user clicks again, we have to close the zoom popup again
                    // on single click.
                    if (!longTapTimer.running) {
//                        console.log("close menu")
                        menus.closeMenu();
                    }
                }
                else
                {
                    if (pip.selected)
                        pip.selected = false
                    else {
                        startDrag = false
                        menus.toggleMenu(); // open the main menu
                    }
                }
            }
            dragged = false
            startDrag = false

            longTapTimer.stop()
        }
    }

    Timer {
        id: longTapTimer
        interval: 500
        repeat: true
        onTriggered: {
            if (running && !desktopMouseArea.dragged && !menus.menuOpen) {
                desktopMouseArea.startDrag = false
                if (!greenbox.mediaControl.busyStore)
                    menus.openTemporary(Menus.ZoomPopupMenu);
            }
        }
    }

    Timer {
        id: longPressArchiveTimer
        interval: 1000
        onTriggered: {
            if (greenbox.appState !== GreenBox.FacetEditView)
                greenbox.system.doNuc()
        }
    }

    Timer {
        id: halfPressTriggerTimer
        interval: 100
        onTriggered: {
            greenbox.system.doOneShotAutoFocus()
        }
    }

    Connections { // get keyboard focus from menu to our focus-scope again after it closes.
        target: menus
        onMenuClosed: {
            // Only get the focus if the menu has it
            var toolbars = greenbox.GetNamedObject("globalToolbars")
            if (toolbars !== null && !toolbars.focus && !toolbars.hasPopupOpen)
                return
//            console.log("Get focus from menuClose")
            desktop.focus = true
        }
    }

    Connections { target: menus.menuItem("edit_cancel")
        onTriggered: {
            cancelEdit()
        }
    }
    Connections { target: menus.menuItem("preview_cancel")
        onTriggered: {
            cancelPreview()
        }
    }
    Connections { target: menus.menuItem("prompt_cancel")
        onTriggered: {
            cancelPrompt()
        }
    }
    Connections { target: menus.menuItem("prompt_voice_cancel")
        onTriggered: {
            cancelPrompt()
        }
    }

    Image{
        source: "../images/Sc_Status_LaserIndicator_Def.png"
        x: flirSystem.laserPosX - (sourceSize.width/2)
        y: flirSystem.laserPosY - (sourceSize.height/2)
        visible: flirSystem.laserActive && flirSystem.laserCalibrated && flirSystem.laserPosX !== -1 && flirSystem.laserPosY !== -1 && greenbox.appState !== GreenBox.FacetEditView
    }

    // Message box
    MessageBox {
        id: editCancelDiag
        title: {
            //: Message header. Question. Displayed when user wants to leave the preview mode via cancel.
            //% "Discard changes?"
            return qsTrId("ID_EDIT_CANCEL_HEADER")
        }
        text: {
            if (cancellingPreview)
            {
                //: Message text body. Displayed when user wants to leave the preview mode via cancel.
                //% "Do you want to discard the changes made to the image?"
                return qsTrId("ID_EDIT_CANCEL_TEXT")
            }
            else
            {
                //: Message text body. Displayed when user wants to leave the edit mode via cancel.
                //% "Do you want to discard the changes made to image %1?"
                return archiveHandler.model.currentGroupName === "" ? qsTrId("ID_EDIT_CANCEL_X_TEXT") : qsTrId("ID_EDIT_CANCEL_X_TEXT").arg(archiveHandler.model.currentGroupName)
            }
        }
        z: 1000
        //: Imperative. Button. Select to save the changes to the image and leave edit mode.
        //% "Save"
        okButtontext: qsTrId("ID_BUTTON_SAVE") +  translation.update
        //: Imperative. Button. Select to discard the image and leave edit mode.
        //% "Discard"
        midButtontext: qsTrId("ID_BUTTON_DISCARD") +  translation.update
        //: Imperative. Button. Select to cancel exiting edit mode.
        //% "Cancel"
        cancelButtontext: qsTrId("ID_BUTTON_CANCEL") +  translation.update
        iconStyle: iconEnum.question
        midButtonVisible: true
        liveBackground: true

        function showDialog() {
            visible = true
            focus = true
            opacity = 1
        }

        function hideDialog() {
            visible = false
            opacity = 0
            focusDesktopFocusGroup()
        }

        onOkSelected: {
            if (greenbox.appState === GreenBox.FacetPreviewView)
                previewHandler.savePreview()
            else
                editHandler.saveEdit()
            hideDialog()
        }
        onCancelSelected: {
            hideDialog()
        }
        onMidSelected: {
            if (greenbox.appState === GreenBox.FacetPreviewView)
                previewHandler.cancelPreview()
            else if (greenbox.appState === GreenBox.FacetPromptView)
                promptHandler.cancelPrompt()
            else
                editHandler.cancelEdit()
            hideDialog()
        }
    }

    MessageBox {
        id: timeLapseDoneDialog
        //: Message header. Displayed when a time-lapse recording ends.
        //% "Time lapse completed"
        title: qsTrId("ID_TIME_LAPSE_COMPLETED") + translation.update
		property bool isVisible: false
        visible: !pogo && isVisible
        opacity: 1
        text: greenbox.mediaControl.timeLapseCompleteMessage
        z: 1000
        hasButtons: false
        iconStyle: iconEnum.info
        liveBackground: true

        function showDialog() {
            isVisible = true
            focus = true
        }

        function hideDialog() {
            isVisible = false
            focusDesktopFocusGroup()
        }

        onCancelSelected: {
            hideDialog()
        }
    }


    Connections { // Listen to when the measurements are no longer active (and set our menus)
        target: measureFuncs
        onSelectionChanged: {
            if (! measureFuncs.hasSelection && !menus.menuOpen) {
                greenbox.adaptMenu()
                // just to be on the safe side, give explicit focus to desktop.
                // This is a no-op in many cases since desktop is a FocusScope,
                // so it would never take away focus from a child of desktop
                // It is needed when deleting measurement function to avoid losing focus.
//                console.log("get focus from selection changed :" + desktop.focus + ", " + desktop.activeFocus)
                desktop.focus = true
            }
//            else if (measureFuncs.hasSelection) {
//                console.log("desktop focus: " + desktop.focus)
//            }
        }
    }
    Connections {
        target: messageboxHandler
        onBoxVisibleChanged: {
            if (messageboxHandler.showing === false)
                focusDesktopFocusGroup()
        }
    }
    Connections {
        target: messageSpinnerHandler
        onSpinnerVisibleChanged: {
            if (messageSpinnerHandler.showing === false)
                focusDesktopFocusGroup()
        }
    }

    Keys.onPressed: {
        if (event.key === keys.trigger) {
            if (halfPressTriggerTimer.running)
                halfPressTriggerTimer.stop()
            triggerActionRequest()
            event.accepted = true
        }
        else if (event.key === keys.up) {
            key_down(GreenBox.UI_KEY_UP,event.isAutoRepeat === true?1:0)
            event.accepted = true
        }
        else if (event.key === keys.down) {
           key_down(GreenBox.UI_KEY_DOWN,event.isAutoRepeat === true?1:0)
            event.accepted = true
        }
        else if (event.key === keys.left) {
            key_down(GreenBox.UI_KEY_LEFT,event.isAutoRepeat === true?1:0)
            event.accepted = true
        }
        else if (event.key === keys.right) {
            key_down(GreenBox.UI_KEY_RIGHT,event.isAutoRepeat === true?1:0)
            event.accepted = true
        }
        else if (event.key === keys.onOff) {
            key_down(GreenBox.UI_KEY_ONOFF,event.isAutoRepeat === true?1:0)
            event.accepted = true
        }
        else if (event.key === keys.archive)
        {
            if (event.isAutoRepeat === false)
                longPressArchiveTimer.start()
            event.accepted = true
        }
        else if (event.key === keys.trigger_half)
        {
            halfPressTriggerTimer.start()
            event.accepted = true
        }
        else if (event.key === keys.program)
        {
//            console.log("Desktop press: event.key === keys.program (" + event.key + ")")
            event.accepted = true
        }
        else if (event.key === keys.zoom)
        {
            greenbox.system.nextZoom()
            event.accepted = true
        }
        else if (event.key === keys.camera)
        {
            if (flirSystem.lampActive)
                flirSystem.lampActive = false
            else
                flirSystem.lampActive = true
            event.accepted = true
        }
        else if (event.key === keys.adjust)
        {
            if (greenbox.appState === GreenBox.FacetEditView ||
                greenbox.appState === GreenBox.FacetPreviewView)
            {
                if (greenbox.scale.interactive)
                {
                    greenbox.scale.interactive = false
                    greenbox.system.doOneShotAutoAdjust()
                }
                else
                    greenbox.scale.interactive = true
            }
            else
            {
                if (greenbox.scale.manual)
                {
                    greenbox.scale.manual = false
                    greenbox.scale.interactive = false
                }
                else
                {
                    greenbox.scale.manual = true
                    greenbox.scale.interactive = true
                }
            }
            event.accepted = true
        }
        else if (event.key === keys.focusIn)
        {
            if (event.isAutoRepeat === false)
                greenbox.system.setFocusSpeed(-40)
            event.accepted = true
        }
        else if (event.key === keys.focusOut)
        {
            if (event.isAutoRepeat === false)
                greenbox.system.setFocusSpeed(40)
            event.accepted = true
        }

        // Debug code - testing set/get of resources
//        else if (event.key === Qt.Key_1) {
//            greenbox.setResource(".image.sysimg.measureFuncs.mbox.1.active", "true")
//        }
//        else if (event.key === Qt.Key_2) {
//            greenbox.setResource(".image.sysimg.measureFuncs.mbox.1.active", "false")
//        }
    }

    Keys.onReleased: {
        if (event.key === keys.archive) {
            if (event.isAutoRepeat === false && !greenbox.mediaControl.busyStore)
            {
                if (greenbox.appState === GreenBox.FacetEditView)
                {
                    cancelEdit()
                }
                else if (longPressArchiveTimer.running)
                {
                    longPressArchiveTimer.stop()

                    if (greenbox.appState === GreenBox.FacetMediaControlState) {
                        console.log("Ignore going into archive while recording/saving.")
                    }
                    else {
                        showArchive()
                    }
                }
            }
            event.accepted = true
        }
        else if (event.key === keys.back && !event.isAutoRepeat)
        {
            if (greenbox.appState === GreenBox.FacetEditView)
                cancelEdit()
            else if (greenbox.appState === GreenBox.FacetPreviewView)
                cancelPreview()
            else if (greenbox.appState === GreenBox.FacetPromptView)
                cancelPrompt()
        }
        else if (event.key === keys.select) {
            if (event.isAutoRepeat === false)
            {
                if (menus.menuOpen || !greenbox.mediaControl.busyStore)
                    menus.toggleMenu()
            }
            event.accepted = true
        }
        else if (event.key === keys.up) {
            key_up(GreenBox.UI_KEY_UP,event.isAutoRepeat === true?1:0)
            tempScale.upKeyPressed()
            event.accepted = true
        }
        else if (event.key === keys.down) {
           key_up(GreenBox.UI_KEY_DOWN,event.isAutoRepeat === true?1:0)
            tempScale.downKeyPressed()
            event.accepted = true
        }
        else if (event.key === keys.left) {
            key_up(GreenBox.UI_KEY_LEFT,event.isAutoRepeat === true?1:0)
            event.accepted = true
        }
        else if (event.key === keys.right) {
            key_up(GreenBox.UI_KEY_RIGHT,event.isAutoRepeat === true?1:0)
            event.accepted = true
        }
        else if (event.key === keys.onOff) {
            key_up(GreenBox.UI_KEY_ONOFF,event.isAutoRepeat === true?1:0)
            event.accepted = true
        }
        else if (event.key === keys.program)
        {
//            console.log("Desktop release: event.key === keys.program (" + event.key + ")")
            event.accepted = true
        }
        else if (event.key === keys.focusIn)
        {
            if (event.isAutoRepeat === false)
                greenbox.system.setFocusSpeed(0)
            event.accepted = true
        }
        else if (event.key === keys.focusOut)
        {
            if (event.isAutoRepeat === false)
                greenbox.system.setFocusSpeed(0)
            event.accepted = true
        }
    }

    Item {
        id: liveControls
        anchors.fill: parent
        visible: greenbox.appState !== GreenBox.FacetArchiveView
        objectName: "desktopTools"

        // FLIR logo
        Image {
            id: logo
            source: "../images/Sc_Logo_FlirHardEdges.png"
            visible: greenbox.system.displayLogo ?
                         (greenbox.appState !== GreenBox.FacetEditView || pogo) ?
                             true : false : false

            // Tilt properties
            rotation: 0
            transformOrigin: Item.BottomLeft
            state: {
                if (greenbox.system.tilt === 0)
                    return "0"
                else if (greenbox.system.tilt === 90)
                    return "90"
                else if (greenbox.system.tilt === 180)
                    return "180"
                else
                    return "270"
            }

            // Tilt states
            states: [
                State {
                    name: "0"
                    PropertyChanges { target: logo; rotation: 0 }
                    PropertyChanges { target: logo; x: grid.leftMargin + grid.irLeftMargin + grid.horizontalSpacing }
                    PropertyChanges { target: logo; y: grid.height - grid.bottomMargin - grid.verticalSpacing - height }
                },
                State {
                    name: "90"
                    PropertyChanges { target: logo; rotation: 90 }
                    PropertyChanges { target: logo; x: grid.leftMargin + grid.irLeftMargin + grid.horizontalSpacing }
                    PropertyChanges { target: logo; y: grid.topMargin + grid.verticalSpacing - height }
                },
                State {
                    name: "180"
                    PropertyChanges { target: logo; rotation: 180 }
                    PropertyChanges { target: logo; x: grid.width - grid.leftMargin - grid.irLeftMargin - grid.horizontalSpacing }
                    PropertyChanges { target: logo; y: -height + grid.topMargin + grid.verticalSpacing }
                },
                State {
                    name: "270"
                    PropertyChanges { target: logo; rotation: 270 }
                    PropertyChanges { target: logo; x: grid.width - grid.horizontalSpacing - grid.rightMargin - grid.irRightMargin }
                    PropertyChanges { target: logo; y: grid.height - grid.bottomMargin - grid.verticalSpacing - height }
                }
            ]
        }

        Pip {id: pip}

        StatusRow { }

        TempScale {
            id: tempScale
        }

        Connections {
            target: greenbox
            onAppStateChanged: {
                if (measureFuncs.hasSelection) {
                    measureFuncs.unselectAll();
                }
            }
        }

        MeasureTools {
            width: grid.width - grid.irLeftMargin - grid.irRightMargin
            height: grid.height
            x: grid.irLeftMargin
        }
    }
}
