import QtQuick 1.1
import se.flir 1.0

Item {

    property bool pipMode: false

    width: grabImageId.width
    height: grabImageId.height

    // box index positions:
    //
    //  (0)----------(2)
    //   |            |
    //   |     (4)    |
    //   |            |
    //  (1)----------(3)
    //
    x: index === 0 || index === 1 ? 0 : index === 4 ? (parent.width - width) / 2 : parent.width - width
    y: index === 0 || index === 2 ? 0 : index === 4 ? (parent.height - height) / 2 : parent.height - width

    MouseArea {

        enabled: boxId.mouseEnabled
        anchors.fill: parent
        anchors.topMargin: -10
        anchors.leftMargin: -10
        anchors.rightMargin: -10
        anchors.bottomMargin: -10

        drag.target: boxId
        drag.minimumX: 0
        drag.minimumY: 0
        drag.maximumX: grid.width - boxId.width - grid.irLeftMargin - grid.irRightMargin - 1
        drag.maximumY: grid.height - boxId.height - 1

        property bool ignoreMouseX: true
        property bool ignoreMouseY: true
        property int grabOffsetX: 0
        property int grabOffsetY: 0

        onMouseXChanged: {
            if (ignoreMouseX === false)  // avoid jerky jump when first grabbing box
            {
                var myMouseX = mouseX - grabOffsetX

                if (index === 0 || index === 1)
                {
                    if (boxId.width - myMouseX > 10)
                    {
                        boxId.x = boxId.x + myMouseX
                        boxId.width = boxId.width - myMouseX
                        if (boxId.x < 0)
                        {
                            boxId.width = boxId.width + boxId.x
                            boxId.x = 0
                        }
                    }
                }
                else if (index === 2 || index === 3)
                {
                    if (boxId.width + myMouseX > 10)
                    {
                        boxId.width = boxId.width + myMouseX
                        if (boxId.width + boxId.x > grid.width - grid.irLeftMargin - grid.irRightMargin)
                            boxId.width = grid.width - grid.irLeftMargin - grid.irRightMargin - boxId.x
                    }
                }

                if (Math.abs(boxId.dragStartWidth - boxId.width) > 5)
                    boxId.moved = true;
            }
            else
            {
                ignoreMouseX = false
                grabOffsetX = mouseX
            }
        }

        onMouseYChanged: {
            if (ignoreMouseY === false) // avoid jerky jump when first grabbing box
            {
                var myMouseY = mouseY - grabOffsetY

                if (index === 0 || index === 2)
                {
                    if (boxId.height - myMouseY > 10)
                    {
                        boxId.y = boxId.y + myMouseY
                        boxId.height = boxId.height - myMouseY
                    }
                }
                else if (index === 1 || index === 3)
                {
                    if (boxId.height + myMouseY > 10)
                        boxId.height = boxId.height + myMouseY
                }

                if (Math.abs(boxId.dragStartHeight - boxId.height) > 5)
                    boxId.moved = true;
            }
            else
            {
                ignoreMouseY = false
                grabOffsetY = mouseY
            }
        }

        onPressAndHold: {
            if (boxId.moved === false)
            {
                if (pipMode === false)
                {
                    menus.activeMenu = Menus.MeasurementMenu
                    menus.toggleMenu()
                }
                drag.target = undefined
            }
        }

        onPressed: {
            if (activeFocus === false && selected === false)
            {
                if (pipMode === false)
                {
                    forceActiveFocus()
                    measureTools.sendEnterMFuncEditRequest(type, resIndex)
                    menus.closeMenu()
                }
                else
                    selected = true
            }
            if (selectionMoveMode)
            {
                if (index !== 4)
                {
                    if (pipMode)
                        boxId.selectionMoveMode = false
                    else
                        measureFuncs.setSelectionMoveMode(false)
                }
            }
            else if (index === 4)
            {
                if (pipMode)
                    boxId.selectionMoveMode = true
                else
                    measureFuncs.setSelectionMoveMode(true)
            }

            boxId.dragStartWidth = boxId.width
            boxId.dragStartHeight = boxId.height

            if (index !== 4)
                drag.target = undefined

            ignoreMouseX = true
            ignoreMouseY = true
        }

        onReleased: {
            boxId.moved = false
            drag.target = boxId

            if (pipMode)
            {
                greenbox.system.setPIPRect(boxId.x, boxId.y,
                                           boxId.x + boxId.width, boxId.y + boxId.height)
            }
            else
                measureFuncs.setPosition(type, resIndex, boxId.x, boxId.y, boxId.width, boxId.height)
        }
    }

    Image {
        id: grabImageId
        source: {
            if (selected)
            {
                if (selectionMoveMode)
                {
                    if (index === 4)
                        return "../images/Sc_MeasureToolPlacedSelected_Marker_Sel.png"
                    else
                        return "../images/Sc_MeasureToolPlacedSelected_Marker_Def.png"
                }
                else
                {
                    if (index === 4)
                        return "../images/Sc_MeasureToolPlacedSelected_Marker_Def.png"
                    else
                        return "../images/Sc_MeasureToolPlacedSelected_Marker_Sel.png"
                }
            }
            else if (pipMode === false)
            {
                if (index === 0)
                    return "../images/Sc_MeasureToolPlaced_BoxTopLeft_Def.png"
                else if (index === 1)
                    return "../images/Sc_MeasureToolPlaced_BoxBottomLeft_Def.png"
                else if (index === 2)
                    return "../images/Sc_MeasureToolPlaced_BoxTopRight_Def.png"
                else if (index === 3)
                    return "../images/Sc_MeasureToolPlaced_BoxBottomRight_Def.png"
                else if (index === 4)
                    return ""
            }
            else
                return ""
        }
    }
}
