import QtQuick 1.1

/**
 *  This QML file is the delegate for the C++ class MultiListMenuItem.
 */

FocusScope {
    width: grid.cellWidth * 5
    height: 153
    id: multiListMenu
    property QtObject model: null


    function close()
    {
        multiListMenu.model.close()
    }


    MouseArea {
        anchors.fill: parent
        // Catch and ignore all mouse clicks in dialog not handled  specifically
    }

    BorderImage {
        anchors.fill: parent
        border { left: 10; top: 10; right: 10; bottom: 10 }
        source: "../images/Bg_MenuMiniWidget_Def.png"

        // Header
        Text {
            id: headerText
            anchors.top: parent.top
            anchors.topMargin: 3*grid.verticalSpacing
            anchors.left: parent.left
            anchors.leftMargin: 4*grid.horizontalSpacing
            font.pixelSize: fonts.smallSize
            font.family: fonts.family
            color: colors.white
            text: multiListMenu.model === null ? "" : multiListMenu.model.headerText + translation.update
        }

        // Blue line
        BorderImage {
            id: toplineImage
            source: "../images/Bg_DialoguePopUp_Line_Def.png"
            border { left: 50; top: 0; right: 50; bottom: 0 }
            anchors.left: parent.left
            anchors.leftMargin: grid.horizontalSpacing
            anchors.right: parent.right
            anchors.rightMargin: grid.horizontalSpacing
            anchors.top: headerText.bottom
            anchors.topMargin: 3*grid.verticalSpacing
        }

        // Row of lists
        Row {

            Repeater {
                id: lists
                model: multiListMenu.model === null ? null : multiListMenu.model.valueLists

                Item{
                    width: multiListMenu.width / multiListMenu.model.logicalColumns * (1 + modelData.rightSpacing)
                    height: multiListMenu.height

                    ValueSelectorListColumn {
                        id: valueList
                        model: modelData.valueList
                        onActivated: multiListMenu.model.acceptChanges()
                        width: multiListMenu.width / multiListMenu.model.logicalColumns
                        height: multiListMenu.height
                        focus: multiListMenu.model.selectedColumn === modelData.column
                        showScrollArrows: multiListMenu.model.selectedColumn === modelData.column
                        Keys.onRightPressed: multiListMenu.model.navigateRight()
                        Keys.onLeftPressed: multiListMenu.model.navigateLeft()
                        currentIndex: modelData.selectedIndex
                        onCurrentIndexChanged: {
                            modelData.selectedIndex = currentIndex
                        }

                        onActiveFocusChanged: {
                            if (focus)
                                multiListMenu.model.selectedColumn = modelData.column
                        }
                    }

                    Text {
                        anchors.left: valueList.right
                        text: modelData.rightDelimiter
                        visible: modelData.rightDelimiter !== ""
                        anchors.verticalCenter: parent.verticalCenter
                        anchors.verticalCenterOffset: 3
                        font.pixelSize: fonts.smallSize
                        font.family: fonts.family
                        color: colors.textNormal
                    }
                }
            }
        }


        // Blue line
        BorderImage {
            id: bottomlineImage
            source: "../images/Bg_DialoguePopUp_Line_Def.png"
            border { left: 50; top: 0; right: 50; bottom: 0 }
            anchors.left: parent.left
            anchors.leftMargin: grid.horizontalSpacing
            anchors.right: parent.right
            anchors.rightMargin: grid.horizontalSpacing
            anchors.bottom: footerText.top
            anchors.bottomMargin: 2*grid.verticalSpacing
        }

        // Footer
        Text {
            id: footerText
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 3*grid.verticalSpacing
            anchors.left: parent.left
            anchors.leftMargin: 4*grid.horizontalSpacing
            font.pixelSize: fonts.smallSize
            font.family: fonts.family
            color: multiListMenu.model === null || multiListMenu.model.selectionValid ? colors.grey : colors.grey // We cant handle text color due to 4:1:1 encoding
            text: multiListMenu.model === null ? "" : multiListMenu.model.footerText + translation.update
        }
    }

    Keys.onReleased: {
        event.accepted = false;
        if (event.key === keys.back) {
            multiListMenu.visible =  false
            event.accepted = true
        }

        if( event.key === keys.up ){
            event.accepted = true
        }
    }

    Keys.onUpPressed: ; // eat those events
    Keys.onDownPressed: ; // eat those events
    Keys.onLeftPressed: ; // eat those events
    Keys.onRightPressed: ; // eat those events
}
