import QtQuick 1.1
import System 1.0
import se.flir 1.0

// ResultTable
// -----------
// The result table.

Rectangle {
    id: root
    property bool tightTempUnit: column.height - 2 <= grid.resultTableRowHeight
    property int numberHiddenResults: 0
    width: resultTable.totalCount === 0 ? 0 :
            (column.labelColumnWidth + column.valueColumnWidth + grid.horizontalSpacing) // that spacing is for the gap between the columns
    visible: cameraHasStaticFocus || greenbox.system.viewMode !== System.VIEW_MODE_VISUAL

    Column {
        id: column
        property int labelColumnWidth: 0
        property int valueColumnWidth: 0
        property int heightLimit: greenbox.system.tilt === 0 || greenbox.system.tilt === 180 ? grid.height : grid.width
        onHeightLimitChanged: repeater.updateColumns()

        Repeater {
            id: repeater

            model: resultTable
            delegate: ResultTableDelegate {
                        width: root.width
                        onColumnWidthsChanged: { // we use this to detect if some large value ended up making the column bigger
                            repeater.updateColumns()
                        }
                    }

            onItemAdded: updateColumns()
            onItemRemoved: updateColumns()

            function updateColumns() {
                var c1 = 0;
                var c2 = 0;
                var bottom = 0
                numberHiddenResults = 0
                for (var i = 0; i < count; ++i) {
                    if (itemAt(i) === null) // not yet created
                        return;
                    c1 = Math.max(c1, itemAt(i).column1Width)
                    c2 = Math.max(c2, itemAt(i).column2Width)

                    // Do not display items that are below the screen
                    bottom += itemAt(i).height
                    if (bottom > column.heightLimit || (i < (count - 1) && bottom + grid.resultTableRowHeight > column.heightLimit))
                    {
                        itemAt(i).visible = false
                        numberHiddenResults++
                    }
                    else if (itemAt(i).visible === false)
                        itemAt(i).visible = true
                }
                column.labelColumnWidth = c1
                column.valueColumnWidth = c2
                // console.debug("c1: "+ c1 + ", c2: " + c2);
            }
        }
    }

    RoundedRect {
        id: moreIndicator
        width: column.width
        height: grid.resultTableRowHeight
        anchors.top: column.bottom
        visible: numberHiddenResults > 0

        Text {
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            font.family: fonts.latinFamily
            font.pixelSize: fonts.miniSize
            color: colors.textNormal
            text: "... (" + numberHiddenResults + ")"
        }
    }

    // Text { text: "totalCount "  + resultTable.totalCount + ", width: " + parent.width + ", tightTempUnit: " + tightTempUnit; y: 100; color: "yellow" }
    // Text { text: "h "  + column.height + ", wantedH:" + grid.resultTableRowHeight; y: 120; color: "yellow" }
}
