import QtQuick 1.1
import se.flir 1.0

Item {
    id: pageRoot
    width: grid.width
    height: grid.height
    state: "live"

    FocusScope {
        id: settingsArea
        width: parent.width
        height: parent.height
        MouseArea { anchors.fill: parent } // catch all mouse action.
        BaseSettingsPage {
            id: settingsPage
            anchors.fill: parent
            model: settings.currentPage
            valueModel: settings.currentPageValues
            pageType: settings.currentPageDelegate
            headerText: settings.currentPageTitle
            footerIcon: settings.footerIcon === "" ? "" : "../images/" + settings.footerIcon
            footerText: qsTrId(settings.footer)
            showArrows: settings.showUpDownArrowsOnCurrent
            currentIndex: settings.selectedIndex
            onBackClicked: parent.goBackOne()
            onItemTriggered: parent.triggerItem(item)
        }

        function goBackOne() {
            // console.debug("prevPageAnim wanted")
            if (settings.atRootItem) {
                settings.closeSettings();
            } else {
                settings.preparePop()
                pageRoot.state = "prevPageAnim"
            }
        }

        function triggerItem(index) {
            if (settings.currentPage.length === 0 || settings.popOnTrigger(index)) { // value-model based data-item, or QObject based with popOnTrigger...
                //console.debug("value-model based triggerItem; index:" + index)
                settings.triggerIndex(index)
                if (settings.popOnTrigger(index))
                    goBackOne()
            }
            else if (settings.currentPage[index].enabled) { // QObject based item.
                //console.debug("QObject based triggerItem; index:" + index)
                settings.triggerIndex(index)
                var next = settings.nextPageFilename(index)
                if (next !== "") {
                    //console.debug("loading page " + next)
                    pageRoot.state = "showSpecialSettingsPage"
                    specialSettingsLoader.source = "" // make sure we unload it when we reenter
                    specialSettingsLoader.source = next
                    specialSettingsLoader.item.model = settings.currentPage[index]
                    specialSettingsLoader.forceActiveFocus()
                    specialSettingsLoader.item.show()
                } else if (settings.isSubmenu(index)) {
                    pageRoot.state = "nextPageAnim"
                } else if (settings.popOnTrigger(index)) {
                    goBackOne()
                }
            } else { // disabled QObject based item
                settings.selectedIndex = index
            }
        }

        Keys.onPressed: {
            if ((event.key == keys.right || event.key == keys.select) && !event.isAutoRepeat) { // first key-repeat event only.
                hiddenServiceMenuTimer.hasStarted = false
            }
        }
        Keys.onReleased: {
            if (event.key === keys.back || event.key == keys.left) {
                if (pageRoot.state == "showSettings")
                    settingsArea.goBackOne();
                event.accepted = true
            }
            else if (event.key === keys.onOff) {
                settings.closeSettings()
                event.accepted = true
            }
            else if (event.key === keys.right || event.key === keys.select) {
                if (event.isAutoRepeat) { // first key-repeat event only.
                    if (!hiddenServiceMenuTimer.hasStarted && !hiddenServiceMenuTimer.running) {
                        hiddenServiceMenuTimer.hasStarted = true;
                        hiddenServiceMenuTimer.start();
                    }
                } else { // last key-repeat event only
                    hiddenServiceMenuTimer.stop();
                    if (pageRoot.state == "showSettings")
                        settingsArea.triggerItem(settingsPage.currentIndex);
                }
                event.accepted = true
            }
            else if (event.key === keys.archive)
            {
                settings.closeSettings()
                // note we do NOT want to accept the event.
            }
            else if (event.key === keys.trigger)
            {
                settings.closeSettings()
                event.accepted = true
            }
        }
    }
    BaseSettingsPage {
        id: settingsAreaShadow
        model: settings.movingPage
        valueModel: settings.movingPageValues
        pageType: settings.movingPageDelegate
        width: parent.width
        height: parent.height
        headerText: settings.movingPageTitle
        showArrows: settings.showUpDownArrowsOnMoving
        footerIcon: settings.movingFooterIcon === "" ? "" : "../images/" + settings.movingFooterIcon
        footerText: qsTrId(settings.movingFooter)
        currentIndex: settings.movingCurrentIndex
    }

    Rectangle { // loader
        width: parent.width
        height: parent.height
        color: colors.background
        id: specialSettingsPage
        visible: false
        Loader {
            anchors.fill: parent
            id: specialSettingsLoader
            x: pageRoot.width
        }
        Connections {
            target: specialSettingsLoader.item
            onHideSignal: exitLoaderSingleShot.start()
        }
        Timer {
            // instead of changing state in the connection above, we use
            // a singleshot. The reason for doing this is that at least on
            // Erla we got a stack-overflow if doing it directly.
            repeat: false
            id: exitLoaderSingleShot
            onTriggered: pageRoot.state = "showSettings"
            interval: 5
        }
    }

    Timer {
        id: hiddenServiceMenuTimer
        property bool hasStarted: false
        interval: 10000
        repeat: false
        onTriggered: {
            if (settings.enterHiddenMenu()) {
                greenbox.system.showHiddenMenu()    // Load any data in the menus. Somewhat hacky, but it is a special case (i.e. don't copy this idea!)
                pageRoot.state = "nextPageAnim"
            }
        }
    }

    states: [
        State {
            name: "live"
            when: greenbox.appState !== GreenBox.FacetSettingsView
            PropertyChanges { target: settingsArea; visible: false }
            PropertyChanges { target: settingsArea; x: pageRoot.width }
            PropertyChanges { target: settingsAreaShadow; visible: false }
            PropertyChanges { target: settingsAreaShadow; x: pageRoot.width }
            StateChangeScript { script: pageRoot.parent.focusDesktopFocusGroup() }
            // StateChangeScript { script: console.debug("state = live") }
        },
        State {
            name: "showSettings"
            when: greenbox.appState === GreenBox.FacetSettingsView
            PropertyChanges { target: settingsArea; visible: true }
            PropertyChanges { target: settingsArea; x: 0 }
            PropertyChanges { target: settingsArea; focus: true }
            PropertyChanges { target: settingsAreaShadow; visible: false }
            PropertyChanges { target: settingsAreaShadow; x: pageRoot.width }
            PropertyChanges { target: specialSettingsPage; x: pageRoot.width }
            PropertyChanges { target: specialSettingsLoader; source: "" }
            StateChangeScript { script: settingsArea.forceActiveFocus() }
            // StateChangeScript { script: console.debug("state = showSettings") }
        },
        State {
            name: "nextPageAnim"
            PropertyChanges { target: settingsArea; visible: true }
            PropertyChanges { target: settingsArea; x: -pageRoot.width }
            PropertyChanges { target: settingsAreaShadow; visible: true }
            PropertyChanges { target: settingsAreaShadow; x: 0 }
            // StateChangeScript { script: console.debug("state = nextPageAnim") }
            StateChangeScript { name: "updateModels"
                script: settings.finishAnimation()
            }
        },
        State {
            name: "prevPageAnim"
            PropertyChanges { target: settingsArea; visible: true }
            PropertyChanges { target: settingsArea; x: pageRoot.width }
            PropertyChanges { target: settingsAreaShadow; visible: true }
            PropertyChanges { target: settingsAreaShadow; x: 0 }
            // StateChangeScript { script: console.debug("state = prevPageAnim") }
            StateChangeScript { name: "updateModels"
                script: settings.finishAnimation()
            }
        },
        State {
            name: "showSpecialSettingsPage"
            PropertyChanges { target: settingsArea; x: -pageRoot.width }
            PropertyChanges { target: settingsAreaShadow; visible: false }
            PropertyChanges { target: specialSettingsPage; x: 0 }
            PropertyChanges { target: specialSettingsPage; visible: true }
        }
    ]

    transitions: [
        Transition {
            from: "live"
            to: "showSettings"
            PropertyAnimation { property: "x"; duration: 200; easing.type: Easing.OutQuad }
        },
        Transition {
            from: "showSettings"
            to: "nextPageAnim"
            SequentialAnimation {
                PropertyAnimation { property: "x"; duration: 200; easing.type: Easing.OutQuad }
                PauseAnimation { duration: 40 }
                ScriptAction { scriptName: "updateModels" }
                PropertyAction { target: pageRoot; property: "state"; value: "showSettings" }
            }
        },
        Transition {
            from: "showSettings"
            to: "prevPageAnim"
            SequentialAnimation {
                PropertyAction { target: settingsAreaShadow; property: "x"; value: -pageRoot.width }
                PropertyAnimation { property: "x"; duration: 200; easing.type: Easing.OutQuad }
                PauseAnimation { duration: 40 }
                ScriptAction { scriptName: "updateModels" }
                PropertyAction { target: pageRoot; property: "state"; value: "showSettings" }
            }
        },
        Transition {
            from: "showSettings"
            to: "showSpecialSettingsPage"
            reversible: true
            SequentialAnimation {
                PropertyAction { target: specialSettingsLoader; property: "source" }
                PropertyAction { target: specialSettingsPage; property: "visible" }
                PropertyAnimation { property: "x"; duration: 200; easing.type: Easing.OutQuad }
                PropertyAction { target: settingsArea; property: "visible" }
            }
        }
    ]
}
