import QtQuick 1.0
import se.flir 1.0

Item {
    width: grid.width
    height: grid.height
    visible: archiveHandler.showSketch

    Sketching {
        id: sketch
        model: archiveHandler.model
        anchors.fill: parent
        onVisibleChanged: if (visible) forceActiveFocus()

        // Most of our user-input comes from the menu. So connect to that.

        Connections { target: menus.menuItem("sketch_cancel")
            onTriggered: {
console.debug("cancel called");
                dialogPane.sourceComponent = cancelSaveMessageBox
                dialogPane.item.show();
            }
        }
        Connections { target: menus.menuItem("sketch_save")
            onTriggered: {
                // TODO save
                archiveHandler.showSketch = false
            }
        }
        Connections { target: menus.menuItem("sketch_color_black")
            onTriggered: {
                sketch.tool = Sketching.Draw
                sketch.color = "black"
            }
        }
        Connections { target: menus.menuItem("sketch_color_white")
            onTriggered: {
                sketch.tool = Sketching.Draw
                sketch.color = "white"
            }
        }
        Connections { target: menus.menuItem("sketch_color_red")
            onTriggered: {
                sketch.tool = Sketching.Draw
                sketch.color = "red"
            }
        }
        Connections { target: menus.menuItem("sketch_color_blue")
            onTriggered: {
                sketch.tool = Sketching.Draw
                sketch.color = "blue"
            }
        }
        Connections { target: menus.menuItem("sketch_color_green")
            onTriggered: {
                sketch.tool = Sketching.Draw
                sketch.color = "green"
            }
        }
        Connections { target: menus.menuItem("sketch_color_yellow")
            onTriggered: {
                sketch.tool = Sketching.Draw
                sketch.color = "yellow"
            }
        }
        Connections { target: menus.menuItem("sketch_stamp_larrow")
            onTriggered: sketch.startStamp(Sketching.StampLeftArrow)
        }
        Connections { target: menus.menuItem("sketch_stamp_rarrow")
            onTriggered: sketch.startStamp(Sketching.StampRightArrow)
        }
        Connections { target: menus.menuItem("sketch_stamp_circle")
            onTriggered: sketch.startStamp(Sketching.StampCircle)
        }
        Connections { target: menus.menuItem("sketch_stamp_cross")
            onTriggered: sketch.startStamp(Sketching.StampCross)
        }
        Connections { target: menus.menuItem("sketch_erase_mode")
            onTriggered: sketch.tool = Sketching.Erase
        }
        Connections { target: menus.menuItem("sketch_clear_all")
            onTriggered: sketch.eraseAll();
        }
        Keys.onPressed: {
            if (sketch.isStamping()) {
                if (event.key === keys.back || event.key == keys.select || event.key == keys.left
                        || event.key == keys.right || event.key == keys.up || event.key == keys.down) {
                    event.accepted = true
                }
            }
        }
        Keys.onReleased: {
            if (sketch.isStamping()) {
                if (event.key === keys.back) {
                    sketch.cancelStamp();
                    event.accepted = true
                } else if (event.key ==  keys.select) {
                    sketch.commitStamp()
                    event.accepted = true
                } else if (event.key == keys.left || event.key == keys.right || event.key == keys.up || event.key == keys.down) {
                    sketch.moveStamp(event.key);
                    event.accepted = true
                }
            }
        }
    }

    Component {
        id: cancelSaveMessageBox

        MessageBox {
            //: Message header. Displayed when user wants to cancel saving her scetch
            //% "Discard Sketch?"
            title: qsTrId("ID_SKETCH_DISCARD_TITLE")

            //: Message body. Displayed when user wants to cancel saving her scetch
            //% "Do you want to discard changes made to image sketch?"
            text: qsTrId("ID_SKETCH_DISCARD_BODY")

            opacity: 1
            iconStyle: iconEnum.info
            midButtonVisible: true
            liveBackground: true

            //: MessageBox button
            //% "Save"
            okButtontext: qsTrId("ID_SKETCH_BUTTON_SAVE")

            //: MessageBox button
            //% "Cancel"
            cancelButtontext: qsTrId("ID_SKETCH_BUTTON_CANCEL")

            //: MessageBox button
            //% "Discard"
            midButtontext: qsTrId("ID_SKETCH_BUTTON_DISCARD")

            onCancelSelected: {
                hide()
            }
            onMidSelected: {
                hide()
                sketch.eraseAll()
                archiveHandler.showSketch = false
            }
            onOkSelected: {
                hide()
                // TODO save
                sketch.eraseAll()
                archiveHandler.showSketch = false
            }
        }
    }

    Loader {
        anchors.fill: parent
        id: dialogPane
    }
}
