import QtQuick 1.1

// KeyDelegate
// ---------------
// Delegate for one Key in the keyboard


Item {
    id: key

    property alias keyWidth : key.width
    height: keyboard.rowHight

    function removeChar() {
        var previousCursorPos = textField.cursorPosition;
        if(textField.cursorPosition > 0) {
            keyboard.model.textValue = keyboard.model.textValue.substring(0,textField.cursorPosition-1) + keyboard.model.textValue.substring(textField.cursorPosition, keyboard.model.textValue.length)
            textField.cursorPosition = previousCursorPos-1;
        }
    }

    function removeAll() {
        var previousCursorPos = textField.cursorPosition;
        if(textField.cursorPosition > 0) {
            keyboard.model.textValue = ""
            textField.cursorPosition = 0;
        }
    }

    function insertChar(character) {
        if(character.length === 0)
            return;
        var previousCursorPos = textField.cursorPosition;
        keyboard.model.textValue = keyboard.model.textValue.substring(0,textField.cursorPosition) + character + keyboard.model.textValue.substring(textField.cursorPosition, keyboard.model.textValue.length)
        textField.cursorPosition = previousCursorPos+1;
    }

    //Key background
    KeyBackground{
        id: keyBackGround
        visible: (control !== "BLANK")
        anchors.fill: key
        anchors.margins: keyboard.keyBackgroundMargin
        keyDown : (mouseArea.pressed)
        dark: (control === "DONE" || control === "?123")
    }

    //Alphanum key foreground
    Text {
        id: secondary_character_surface
        visible: (control === "")
        anchors.left: parent.left;
        anchors.top: parent.top;
        anchors.leftMargin: keyboard.keySecondaryMargin
        anchors.topMargin: keyboard.keySecondaryMarginTop
        color: keyboard.secondaryCharacterColor
        font.pixelSize: keyboard.secondaryCharacterSize
        text:  special1Visible ? special2 : special1
    }
    Text {
        id: primary_character_surface
        visible: (control === "")
        anchors.bottom: parent.bottom
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.margins: keyboard.keyPrimaryMargin
        font.bold: true
        color: keyboard.primaryCharacterColor
        font.pixelSize: keyboard.primaryCharacterSize
        text: lowerCaseVisible ? lower_case : (upperCaseVisible ?  upper_case : (special1Visible ? special1 : special2))
    }



    // Text based control key foreground
    Text {
        id: text_control_key
        visible: (control !== "")
        text: (control === "?123" ? ((keyboardShiftState & keyboardShiftStates.alternate) ? "ABC" : "?123") : (control === "DONE" ? "Done" : ( control === "SHIFT" ? ((keyboardShiftState & keyboardShiftStates.alternate && !(keyboardShiftState & keyboardShiftStates.shifted)) ? "1/2" : ( (keyboardShiftState & keyboardShiftStates.alternate && keyboardShiftState & keyboardShiftStates.shifted) ? "2/2" : "")  ) : "")))
        anchors.centerIn: parent
        font.bold: true
        font.pixelSize: keyboard.primaryCharacterSize
        color: keyboard.secondaryCharacterColor
    }

    // Image based control key foreground
    Image {
        id: image_control_key
        visible: (control !== "")
        anchors.centerIn: parent
        source: (control === "BACKSPACE" ? "../../images/Kb_Ic_Backspace_Def.png" : (control === "SPACE" ? "../../images/Kb_Ic_Space_Def.png" : (control == "ENTER" ? "../../images/Kb_Ic_Enter_Def.png" : (control === "SHIFT" ? ((!(keyboardShiftState & keyboardShiftStates.alternate)) ?  "../../images/Kb_Ic_Shift_Def.png" : ""):""))))
    }

    // Caps lock indicator
    Image {
        id: control_caps_lock
        visible: (control === "SHIFT")
        source: (!(keyboardShiftState & keyboardShiftStates.caps_lock) && !(keyboardShiftState & keyboardShiftStates.alternate)) ? "../../images/Kb_Btn_ShiftIndicator_Def.png" : ((keyboardShiftState & keyboardShiftStates.caps_lock) && !(keyboardShiftState & keyboardShiftStates.alternate) ? "../../images/Kb_Btn_ShiftIndicator_Sel.png" : "" )
        anchors.margins: keySecondaryMargin
        anchors.top: parent.top;
        anchors.left: parent.left;
    }

    Component.onCompleted: popup_loader.sourceComponent = popup_component
    Loader
    {
        id:popup_loader
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        anchors.right: parent.right
        height: parent.height * 1.5
    }

    Component{ id: popup_component
        Item{
            id:popup

            visible: mouseArea.pressed && (control == "")
            width: key.width * 1.5
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: -(keyBackGround.height / 2 + parent.height / 5)
            anchors.horizontalCenter: parent.horizontalCenter
            z:1


            KeyBackground{
                id:popup_background
                anchors.fill: parent
                anchors.margins: keyboard.keyBackgroundMargin
                popup: true
                longPressActive: mouseArea.longPress
            }

             Text {
                id: popup_foreground
                text: (!(keyboardShiftState & keyboardShiftStates.shifted) && !(keyboardShiftState & keyboardShiftStates.alternate) && !mouseArea.longPress) ? lower_case :((keyboardShiftState & keyboardShiftStates.shifted && !(keyboardShiftState & keyboardShiftStates.alternate)  && !mouseArea.longPress) ? upper_case : (((!(keyboardShiftState & keyboardShiftStates.shifted) && keyboardShiftState & keyboardShiftStates.alternate && !mouseArea.longPress) || (keyboardShiftState & keyboardShiftStates.shifted && keyboardShiftState & keyboardShiftStates.alternate && mouseArea.longPress) || ( !(keyboardShiftState & keyboardShiftStates.alternate) && mouseArea.longPress)) ? special1 : (special2)))
                anchors.centerIn: parent;
                color: keyboard.primaryCharacterColor
                font.bold: true
                font.pixelSize: keyboard.primaryCharacterSize
            }
        }
    }


    // Touch handling
    MouseArea {
        id: mouseArea
        anchors.fill: parent
        anchors.margins: keyMouseAreaMargin

        // properties
        property bool longPress : false

        onReleased: {
            // Case: Clicking printable key
            if(control === "") {
                if(longPress && !(keyboardShiftState & keyboardShiftStates.alternate)) {
                    insertChar(special1)
                }
                else if(longPress && keyboardShiftState & keyboardShiftStates.alternate) {
                    if(keyboardShiftState & keyboardShiftStates.shifted)
                        insertChar(special1)
                    else
                        insertChar(special2)
                }
                else if(keyboardShiftState & keyboardShiftStates.alternate) {
                    if(keyboardShiftState & keyboardShiftStates.shifted) {
                        insertChar(special2)
                    }
                    else {
                        insertChar(special1)
                    }
                }
                else {
                    if(keyboardShiftState & keyboardShiftStates.shifted) {
                        insertChar(upper_case)
                    }
                    else {
                        insertChar(lower_case)
                    }
                }
                if( keyboardShiftState & keyboardShiftStates.shifted
                        && !(keyboardShiftState & keyboardShiftStates.caps_lock)
                        && !(keyboardShiftState & keyboardShiftStates.alternate)) {
                    keyboardShiftState &=  ~keyboardShiftStates.shifted ;
                }
            }
            // Case: Clicking control key
            else if(control === "SPACE") {
                insertChar(" ")
            }
            else if(control === "ENTER") {
                insertChar("\n")
            }
            else if(control === "BACKSPACE" && keyboard.model.textValue.length > 0) {
                if(!longPress) {
                    removeChar()
                }
                // Already handled in onPressAndHold, just exit
            }
            else if(control === "?123") {
                keyboardShiftState ^= keyboardShiftStates.alternate;
                keyboardShiftState &= ~(keyboardShiftStates.shifted | keyboardShiftStates.caps_lock);
            }
            else if(control === "DONE")
            {
                softKeyboardPage.model.result = true
                softKeyboardPage.hide()
            }
            else if(control === "SHIFT") {
                if(!(keyboardShiftState & keyboardShiftStates.alternate)) {
                    // Current state: alpha layout
                    if(longPress) {
                        // Already handled in onPressAndHold, just exit
                    }
                    else if(keyboardShiftState & keyboardShiftStates.shifted) {
                        if(!(keyboardShiftState & keyboardShiftStates.caps_lock)) {
                            // If shift go to caps lock
                            keyboardShiftState |= keyboardShiftStates.caps_lock;
                        }
                        else {
                            // If caps lock go to normal (unshifted)
                            keyboardShiftState &=  ~(keyboardShiftStates.shifted | keyboardShiftStates.caps_lock);
                        }
                    }
                    else {
                        // If normal(unshifted) go to shifted
                        keyboardShiftState |= keyboardShiftStates.shifted;
                    }
                }
                else {
                    // Current state: numeric and special char layout -> toggle shift
                    keyboardShiftState ^= keyboardShiftStates.shifted;
                }

            }
            longPress = false
        }

        onPressAndHold: {
            longPress = true;

            if(longPress && control === "SHIFT" && !(keyboardShiftState & keyboardShiftStates.alternate)) {
                if(keyboardShiftState & keyboardShiftStates.caps_lock) {
                    keyboardShiftState &=  ~(keyboardShiftStates.shifted | keyboardShiftStates.caps_lock);
                }
                else {
                    keyboardShiftState |= keyboardShiftStates.caps_lock | keyboardShiftStates.shifted;
                }
            }
            if(longPress && control === "BACKSPACE") {
                removeAll()
            }

        }
    }
}
