import QtQuick 1.1
import se.flir 1.0

// Spot
// ----
// A spot.

Item {

    //anchors.fill: imageID
    id: spotId
    width: crossHair.width
    height: crossHair.height
    x: toolX - width / 2
    y: toolY - height / 2
    property bool moved: false
    property int dragStartX: 0
    property int dragStartY: 0

    // Properties needed if binding is broken by joystick navigation
    property int myX: toolX - width / 2
    property int myY: toolY - width / 2
    onMyXChanged: {
        if (keyFilterTimer.running === false)
            x = myX
    }
    onMyYChanged: {
        if (keyFilterTimer.running === false)
            y = myY
    }

    Timer {
        id: keyFilterTimer
        interval: 2000
        onTriggered:
        {
            x = toolX - width / 2
            y = toolY - height / 2
        }
    }

    // Debug output
//  onXChanged: console.log("Spot.x: " + x)
//  onYChanged: console.log("Spot.y: " + y)

    property bool mySelection: selected
    onMySelectionChanged: {
        if (selected === true && activeFocus === false) {
            forceActiveFocus()
            keepFocusTimer.start()
            measureTools.sendEnterMFuncEditRequest(type, resIndex)
        } else if (selected === false) {
            menus.closeMenu()
        }
    }

    Timer {
        id: keepFocusTimer
        interval: 300
        onTriggered: {
            if (selected === true && activeFocus === false) {
                forceActiveFocus()
            }
        }
    }

    // Selection marker
    Rectangle {
        anchors.verticalCenter: parent.verticalCenter
        anchors.horizontalCenter: parent.horizontalCenter
        color: colors.mfuncHighlight
        visible: selected
        width: 18
        height: 18
    }

    Item {
        id: crossHair
        width: 30 * greenbox.system.zoom
        height: 30 * greenbox.system.zoom

        CrossHair{
            width: crossHair.width
            height: crossHair.height
        }
    }

    // The ID
    Rectangle {
        id: idRect
        color: colors.transparent
        anchors.right: parent.right
        anchors.top: parent.top
        width: idLabel.width + grid.horizontalSpacing
        height: idLabel.height
        visible: measureFuncs.spotCount > 1 ? true : false

        RoundedRect {
            anchors.fill: parent
        }

        Text {
            id: idLabel
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            text: resIndex + 1
            color: colors.textNormal
        }
    }

    // Local paramater indicator
    Rectangle {
        id: paramsRect
        color: colors.transparent
        anchors.left: idRect.right
        anchors.leftMargin: 1
        anchors.top: parent.top
        width: paramsLabel.width + grid.horizontalSpacing
        height: paramsLabel.height
        visible: localParams

        RoundedRect {
            anchors.fill: parent
        }

        Text {
            id: paramsLabel
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            //: One letter abbreviation of 'parameters'. It is used as a flag indicating that a measurement tool has parameter settings that differs from default.
            //% "P"
            text: qsTrId("ID_MEASURE_LOCAL_PARAMS_P") + translation.update
            color: colors.textNormal
        }
    }

    MouseArea {

        anchors.fill: parent
        anchors.topMargin: -10
        anchors.leftMargin: -10
        anchors.rightMargin: -10
        anchors.bottomMargin: -10

        drag.target: parent
        drag.minimumX: -parent.width / 2
        drag.minimumY: -parent.height / 2
        drag.maximumX: grid.width - grid.irLeftMargin - grid.irRightMargin - parent.width / 2 - 1
        drag.maximumY: grid.height - parent.height / 2 - 1

        onMouseXChanged: {
            if (Math.abs(dragStartX - parent.x) > 5)
                moved = true
        }

        onMouseYChanged: {
            if (Math.abs(dragStartY - parent.y) > 5)
                moved = true
        }

        onPressAndHold: {
            if (moved === false)
            {
                menus.toggleMenu()
                drag.target = undefined
            }
        }

        onPressed: {
            if (activeFocus === false && selected === false)
            {
                forceActiveFocus()
                measureTools.sendEnterMFuncEditRequest(type, resIndex)
                menus.navigateCancel()
            }

            dragStartX = parent.x
            dragStartY = parent.y
        }

        onReleased: {
            moved = false
            drag.target = parent
            measureFuncs.setPosition(type, resIndex, parent.x + parent.width / 2, parent.y + parent.height / 2, -1, -1)
        }
    }

    Keys.onPressed: {
        if (selected)
        {
            if (event.key === keys.up) {
                event.accepted = true
                y = Math.max(y - 10, -height / 2)
            }
            else if (event.key === keys.down) {
                event.accepted = true
                y = Math.min(y + 10, grid.height - height / 2 - 1)
            }
            else if (event.key === keys.left) {
                event.accepted = true
                x = Math.max(x - 10, -width / 2)
            }
            else if (event.key === keys.right) {
                event.accepted = true
                x = Math.min(x + 10, grid.width - grid.irLeftMargin - grid.irRightMargin - width / 2 - 1)
            }
        }
    }
    Keys.onReleased: {
        if (selected)
        {
            if (event.key === keys.up || event.key === keys.down ||
                event.key === keys.left || event.key === keys.right)
            {
                event.accepted = true
                keyFilterTimer.restart()
                if (event.isAutoRepeat === false)
                    measureFuncs.setPosition(type, resIndex, x + width / 2, y + height / 2, -1, -1)
            }
        }
    }
}
