import QtQuick 1.1
import System 1.0
import se.flir 1.0

//! Temperature Scale
//!
//! The scale with the palette gradient on the side of the screen.
//! @ingroup QML

Item {
    id: scaleAndIsothermItem

    // TODO: Compute this some more elaborate way
    width: fonts.smallSize * 2 + grid.horizontalSpacing * 2
    visible: greenbox.appState !== GreenBox.FacetArchiveView

    Component.onCompleted: {
        greenbox.scale.interactive = manualInteraction;
        updateLaserPosDelay.restart()
    }
    onRotationChanged: updateLaserPosDelay.restart()

    // Tilt properties
    rotation: 0
    transformOrigin: Item.TopRight
    state: {
        if (greenbox.system.tilt === 0)
            return "0"
        else if (greenbox.system.tilt === 90)
            return "90"
        else if (greenbox.system.tilt === 180)
            return "180"
        else
            return "270"
    }

    function upKeyPressed() {
        keyUpFeedbackTimer.restart()
        keyDownFeedbackTimer.stop()
    }

    function downKeyPressed() {
        keyDownFeedbackTimer.restart()
        keyUpFeedbackTimer.stop()
    }

    Timer {
        id: keyUpFeedbackTimer
        interval: 500
        repeat: false
    }
    Timer {
        id: keyDownFeedbackTimer
        interval: 500
        repeat: false
    }

    Item {
        id: scaleItem
        anchors.fill: parent
        visible: greenbox.system.viewMode !== System.VIEW_MODE_VISUAL

        // The gradient visualising the scale
        ScaleImage {
            id: scaleGradient
            anchors.right: parent.right
            anchors.top: upperRow.bottom
            anchors.topMargin: grid.verticalSpacing
            anchors.bottom: lowerRow.top
            anchors.bottomMargin: grid.verticalSpacing
            width: grid.horizontalSpacing * 4

            // The visual above color
            Rectangle {
                id: isothermAboveLevel
                anchors.left: scaleGradient.left
                anchors.top: scaleGradient.top
                anchors.right: scaleGradient.right

                height: Math.max(scaleGradient.height - ((scaleGradient.height * greenbox.isotherm.pos ) / 100), 2)

                color: greenbox.isotherm.color
                visible: greenbox.isotherm.type === 1 ||
                         (greenbox.isotherm.type === 5 && greenbox.isotherm.actualType === 1)
            }

            // The visual below color
            Rectangle {
                id: isothermBelowLevel
                anchors.left: scaleGradient.left
                anchors.bottom: scaleGradient.bottom
                anchors.right: scaleGradient.right

                height: Math.max((scaleGradient.height * greenbox.isotherm.pos ) / 100, 2)

                color: greenbox.isotherm.color
                visible: greenbox.isotherm.type === 2 || greenbox.isotherm.type === 4 ||
                         (greenbox.isotherm.type === 5 && greenbox.isotherm.actualType === 2)
            }

            // The visual interval color
            Rectangle {
                id: isothermIntervalLevel
                anchors.left: scaleGradient.left
                anchors.top: scaleGradient.top
                anchors.topMargin: scaleGradient.height - ((scaleGradient.height * greenbox.isotherm.pos ) / 100)
                anchors.right: scaleGradient.right

                height: Math.min((scaleGradient.height * greenbox.isotherm.lowPos ) / 100,
                                 (scaleGradient.height * greenbox.isotherm.pos ) / 100)

                color: greenbox.isotherm.color
                visible: greenbox.isotherm.type === 3
            }

            // Isotherm interaction marker (arrows)
            Image {
                id: arrowsID
                source: y < 0 ? "../images/Sc_Status_ScaleArrowDown_Sel.png" :
                                y > scaleGradient.height - height ? "../images/Sc_Status_ScaleArrowUp_Sel.png" :
                                                                        "../images/Sc_Status_ScaleArrowUpDown_Sel.png"
                visible: greenbox.isotherm.type === 0 ?
                             false :
                             (greenbox.scale.interactionMode === Scale.InteractionObjectIsotherm ||
                              greenbox.scale.interactionMode === Scale.InteractionObjectIsothermMax ||
                              greenbox.scale.interactionMode === Scale.InteractionObjectIsothermMin) ? true : false
                anchors.horizontalCenter: scaleGradient.horizontalCenter
                y: greenbox.scale.interactionMode === Scale.InteractionObjectIsothermMin ?
                       isothermIntervalLevel.y + isothermIntervalLevel.height - height / 2 :
                       Math.min(
                           Math.max(scaleGradient.height - ((scaleGradient.height * greenbox.isotherm.pos ) / 100) - height / 2,
                                -height / 3), scaleGradient.height - 2 * height / 3)
            }

            // Gradient border
            BorderImage {
                anchors.fill: scaleGradient
                anchors.topMargin: -grid.verticalSpacing
                anchors.bottomMargin: -grid.verticalSpacing
                border { left: grid.unit; top: grid.unit; right: grid.unit; bottom: grid.unit }
                source: greenbox.scale.manual?"../images/Bg_Scale_OutlineLocked_Def.png":"../images/Bg_Scale_Outline_Def.png"
            }

            MouseArea {
                id: scaleGradientMouseArea
                anchors.fill: scaleGradient
                anchors.leftMargin: -scaleGradient.width // increase area to accound for poor touch screen calibration
                onPressed: {
                    menus.closeMenu()
                    if (greenbox.appState === GreenBox.FacetEditView ||
                        greenbox.appState === GreenBox.FacetPreviewView)
                    {
                        if (greenbox.scale.interactive)
                        {
                            greenbox.scale.interactive = false
                            greenbox.system.doOneShotAutoAdjust()
                        }
                        else
                            greenbox.scale.interactive = true
                    }
                    else
                    {
                        if (greenbox.scale.manual)
                        {
                            greenbox.scale.manual = false
                            greenbox.scale.interactive = false
                        }
                        else
                        {
                            greenbox.scale.manual = true
                            greenbox.scale.interactive = true
                        }
                    }
                }
            }
        }

        // The background of the upper label
        RoundedRect {
            id: upperRow
            anchors.right: parent.right
            anchors.top: parent.top
            width: upperContent.width + grid.horizontalSpacing * 4
            height: grid.resultTableRowHeight
            selected: (pogo || greenbox.scale.manual === false || greenbox.scale.interactive === false) ?
                          false :
                          (greenbox.scale.interactionMode === Scale.InteractionObjectLevel ||
                           greenbox.scale.interactionMode === Scale.InteractionObjectMax) ? true : false

            Row {
                anchors.centerIn: upperRow
                id: upperContent
                spacing: grid.horizontalSpacing
                height: grid.resultTableRowHeight

                // Arrows
                Item
                {
                    anchors.verticalCenter: parent.verticalCenter
                    width: topKeyUpIndicator.visible ? topKeyUpIndicator.width : topKeyDownIndicator.visible ? topKeyDownIndicator.width : 0
                    height: parent.height

                    // Upper up-arrow
                    Image {
                        id: topKeyUpIndicator
                        anchors.verticalCenter: parent.verticalCenter
                        visible: !pogo && greenbox.scale.manual && (!keyDownFeedbackTimer.running || upperRow.selected === false) &&
                                 ((greenbox.appState !== GreenBox.FacetEditView && greenbox.appState !== GreenBox.FacetPreviewView) || greenbox.scale.interactive)
                        width: pogo ? 0 : implicitWidth
                        source: greenbox.scale.interactive === false ?
                                    "../images/Sc_Status_Locked_Def.png" :
                                    (greenbox.scale.interactionMode === Scale.InteractionObjectLevel ||
                                     greenbox.scale.interactionMode === Scale.InteractionObjectMax) ?
                                        "../images/Sc_Status_ScaleArrowUp_Act.png" :
                                        "../images/Sc_Status_ScaleArrowUp_Def.png"
                    }

                    // Upper down-arrow
                    Image {
                        id: topKeyDownIndicator
                        anchors.verticalCenter: parent.verticalCenter
                        visible: !pogo && greenbox.scale.manual && (!keyUpFeedbackTimer.running || upperRow.selected === false) &&
                                 ((greenbox.appState !== GreenBox.FacetEditView && greenbox.appState !== GreenBox.FacetPreviewView) || greenbox.scale.interactive)
                        width: pogo ? 0 : implicitWidth
                        source: greenbox.scale.interactive === false ?
                                  "../images/Sc_Status_Locked_Def.png" :
                                  (greenbox.scale.interactionMode === Scale.InteractionObjectLevel ||
                                   greenbox.scale.interactionMode === Scale.InteractionObjectMax) ?
                                         "../images/Sc_Status_ScaleArrowDown_Act.png" :
                                         "../images/Sc_Status_ScaleArrowDown_Def.png"
                    }
                }

                // The upper label
                Text {
                    id: upperLabel
                    anchors.verticalCenter: parent.verticalCenter

                    font.family: fonts.latinFamily
                    font.pixelSize: fonts.smallSize
                    font.bold: true
                    color: upperRow.selected ? "black" : colors.textFocused
                    text: greenbox.scale.levelHigh
                }
            }
        }

        // The background of the lower label
        RoundedRect {
            id: lowerRow
            anchors.right: parent.right
            anchors.bottom: parent.bottom
            width: lowerContent.width + grid.horizontalSpacing * 4
            height: grid.resultTableRowHeight
            selected: (pogo || greenbox.scale.manual === false || greenbox.scale.interactive === false) ?
                          false :
                          (greenbox.scale.interactionMode === Scale.InteractionObjectLevel ||
                           greenbox.scale.interactionMode === Scale.InteractionObjectMin) ? true : false

            Row {
                anchors.centerIn: lowerRow
                id: lowerContent
                spacing: grid.horizontalSpacing
                height: grid.resultTableRowHeight

                // Arrows
                Item
                {
                    anchors.verticalCenter: parent.verticalCenter
                    width: bottomKeyUpIndicator.visible ? bottomKeyUpIndicator.width : bottomKeyDownIndicator.visible ? bottomKeyDownIndicator.width : 0
                    height: parent.height

                    // Lower up-arrow
                    Image {
                        id: bottomKeyUpIndicator
                        anchors.verticalCenter: parent.verticalCenter
                        visible: !pogo && greenbox.scale.manual && (!keyDownFeedbackTimer.running || lowerRow.selected === false) &&
                                 ((greenbox.appState !== GreenBox.FacetEditView && greenbox.appState !== GreenBox.FacetPreviewView) || greenbox.scale.interactive)
                        width: pogo ? 0 : implicitWidth
                        source: greenbox.scale.interactive === false ?
                                    "../images/Sc_Status_Locked_Def.png" :
                                    (greenbox.scale.interactionMode === Scale.InteractionObjectLevel ||
                                     greenbox.scale.interactionMode === Scale.InteractionObjectMin) ?
                                     "../images/Sc_Status_ScaleArrowUp_Act.png" :
                                     "../images/Sc_Status_ScaleArrowUp_Def.png"
                    }

                    // Lower down-arrow
                    Image {
                        id: bottomKeyDownIndicator
                        anchors.verticalCenter: parent.verticalCenter
                        visible: !pogo && greenbox.scale.manual && (!keyUpFeedbackTimer.running || lowerRow.selected === false) &&
                                 ((greenbox.appState !== GreenBox.FacetEditView && greenbox.appState !== GreenBox.FacetPreviewView) || greenbox.scale.interactive)
                        width: pogo ? 0 : implicitWidth
                        source: greenbox.scale.interactive === false ?
                                    "../images/Sc_Status_Locked_Def.png" :
                                    (greenbox.scale.interactionMode === Scale.InteractionObjectLevel ||
                                     greenbox.scale.interactionMode === Scale.InteractionObjectMin) ?
                                     "../images/Sc_Status_ScaleArrowDown_Act.png" :
                                     "../images/Sc_Status_ScaleArrowDown_Def.png"
                    }
                }

                // The lower label
                Text {
                    id: lowerLabel
                    anchors.verticalCenter: parent.verticalCenter

                    font.family: fonts.latinFamily
                    font.pixelSize: fonts.smallSize
                    font.bold: true
                    color: lowerRow.selected ? "black" : colors.textFocused
                    text: greenbox.scale.levelLow
                }
            }
        }
    }

    // The isotherm indicator
    RoundedRect {

        id: isotherm
        anchors.bottom: parent.bottom
        x: scaleItem.visible ? parent.width - width - upperRow.width - grid.horizontalSpacing : parent.width - width

        width: isothermContent.width + grid.horizontalSpacing * 4
        height: isothermContent.height

        visible: greenbox.isotherm.type === 0 ? false : greenbox.system.viewMode === System.VIEW_MODE_VISUAL ? false : true
        selected: greenbox.scale.interactionMode === Scale.InteractionObjectIsotherm ? true : false

        // Isotherm value(s)
        Row {
            id: isothermContent
            spacing: grid.horizontalSpacing
            height: grid.resultTableRowHeight
            anchors.centerIn: isotherm

            Rectangle {
                anchors.verticalCenter: parent.verticalCenter
                color: greenbox.isotherm.color
                width: 10
                height: 10
            }

            Item {
                width: isoLowVal.text === "" ? 0: isoLowVal.width + isoSymbol.width
                height: grid.resultTableRowHeight

                Rectangle {
                    anchors.verticalCenter: parent.verticalCenter
                    visible: (greenbox.scale.interactionMode === Scale.InteractionObjectIsotherm ||
                              greenbox.scale.interactionMode === Scale.InteractionObjectIsothermMin) ? true : false
                    color: "white"
                    width: isoLowVal.width + 3
                    height: parent.height - 4
                    radius: 2
                }

                Image {
                    id: isoSymbol
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.left: parent.left
                    width: visible ? undefined : 0
                    visible: greenbox.isotherm.type !== 3 ? true : false
                    source: (greenbox.isotherm.type === 1 || (greenbox.isotherm.type === 5 && greenbox.isotherm.actualType === 1)) ?
                                (isotherm.selected ? "../images/Sc_Arrow_AboveValue_Act.png" : "../images/Sc_Arrow_AboveValue_Def.png") :
                                (isotherm.selected ? "../images/Sc_Arrow_BelowValue_Act.png" : "../images/Sc_Arrow_BelowValue_Def.png")
                }

                Text {
                    id: isoLowVal
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.left: isoSymbol.right
                    anchors.leftMargin: 2
                    font.family: fonts.latinFamily
                    font.pixelSize: fonts.smallSize
                    font.bold: true
                    color: (greenbox.scale.interactionMode === Scale.InteractionObjectIsotherm ||
                            greenbox.scale.interactionMode === Scale.InteractionObjectIsothermMin) ?
                               "black" : colors.textFocused
                    text: {
                        if (greenbox.isotherm.type === 1)
                            return greenbox.isotherm.level
                        else if (greenbox.isotherm.type === 2)
                            return greenbox.isotherm.level
                        else if (greenbox.isotherm.type === 3)
                            return greenbox.isotherm.lowLevel + (greenbox.scale.interactionMode === Scale.InteractionObjectIsothermMax ? " - " : "")
                        else if (greenbox.isotherm.type === 4)
                            return greenbox.isotherm.level
                        else if (greenbox.isotherm.type === 5)
                        {
                            if (greenbox.isotherm.actualType === 1)
                                return greenbox.isotherm.level
                            else
                                return greenbox.isotherm.level
                        }
                        else
                            return ""       // Default to avoid warnings during startup
                    }
                }
            }

            Item {
                width: greenbox.isotherm.type === 3 ? isoHighVal.width : 0
                height: grid.resultTableRowHeight

                Rectangle {
                    anchors.verticalCenter: parent.verticalCenter
                    visible: greenbox.scale.interactionMode === Scale.InteractionObjectIsothermMax ? true : false
                    color: "white"
                    width: isoHighVal.width + 2
                    height: parent.height - 4
                    radius: 2
                    x: -1
                }

                Text {
                    id: isoHighVal
                    anchors.verticalCenter: parent.verticalCenter
                    font.family: fonts.latinFamily
                    font.pixelSize: fonts.smallSize
                    font.bold: true
                    color: greenbox.scale.interactionMode === Scale.InteractionObjectIsothermMax ? "black" : colors.textFocused
                    visible: greenbox.isotherm.type === 3 ? true : false
                    text: (greenbox.scale.interactionMode === Scale.InteractionObjectIsothermMax ? "" : " - ") + greenbox.isotherm.level
                }
            }
        }
    }

    property bool inited: false     // Needed to avoid displaying text at startup

    Connections {
        target: greenbox.scale
        onInteractionModeChanged:   {
            if (!inited)
            {
                inited = true
                return
            }

             if (greenbox.scale.interactionMode === Scale.InteractionObjectLevel)
                 //: Noun. The current keyboard state is to adjust the max and min temperature thresholds of the image color distribution (move the entire histogram).
                 //% "Max+Min"
                  greenbox.emitShowNotification("../images/Sc_Arrow_BelowValue_Def.png", qsTrId("ID_ADJUST_STATE_SCALE_LEVEL"), "../images/Sc_Arrow_AboveValue_Def.png")
             else if (greenbox.scale.interactionMode === Scale.InteractionObjectMax)
                 //: Noun. The current keyboard state is to adjust image max temperature threshold of the color distribution (move the top end of the histogram).
                 //% "Max"
                 greenbox.emitShowNotification("../images/Sc_Arrow_BelowValue_Def.png", qsTrId("ID_ADJUST_STATE_SCALE_MAX"), "../images/Sc_Arrow_AboveValue_Def.png")
             else if (greenbox.scale.interactionMode === Scale.InteractionObjectMin)
                 //: Noun. The current keyboard state is to adjust image min temperature threshold of the color distribution (move the bottom end of the histogram).
                 //% "Min"
                 greenbox.emitShowNotification("../images/Sc_Arrow_BelowValue_Def.png", qsTrId("ID_ADJUST_STATE_SCALE_MIN"), "../images/Sc_Arrow_AboveValue_Def.png")
             else if (greenbox.scale.interactionMode === Scale.InteractionObjectIsotherm)
             {
                 if (greenbox.isotherm.actualType === 1)
                 {
                     //: Noun. The current keyboard state is to adjust the isotherm above level (temperature colorizing threshold). Should be translated to reflect ID_ABOVE.
                     //% "Above alarm"
                     greenbox.emitShowNotification("../images/Sc_Arrow_BelowValue_Def.png", qsTrId("ID_ADJUST_STATE_ISOTHERM_ABOVE"), "../images/Sc_Arrow_AboveValue_Def.png")
                 }
                 else
                 {
                     //: Noun. The current keyboard state is to adjust the isotherm below level (temperature colorizing threshold). Should be translated to reflect ID_BELOW.
                     //% "Below alarm"
                     greenbox.emitShowNotification("../images/Sc_Arrow_BelowValue_Def.png", qsTrId("ID_ADJUST_STATE_ISOTHERM_BELOW"), "../images/Sc_Arrow_AboveValue_Def.png")
                 }
             }
             else if (greenbox.scale.interactionMode === Scale.InteractionObjectIsothermMax)
                 //: Noun. The current keyboard state is to adjust the maximum interval isotherm level (in cases where there are two limits). 'Iso' is short for isotherm. 'max' is short for maximum.
                 //% "Interval.max"
                 greenbox.emitShowNotification("../images/Sc_Arrow_BelowValue_Def.png", qsTrId("ID_ADJUST_STATE_ISTOTHERM_MAX"), "../images/Sc_Arrow_AboveValue_Def.png")
             else if (greenbox.scale.interactionMode === Scale.InteractionObjectIsothermMin)
                 //: Noun. The current keyboard state is to adjust the minimum interval isotherm level (in cases where there are two limits). 'Iso' is short for isotherm. 'min' is short for minimum.
                 //% "Interval.min"
                 greenbox.emitShowNotification("../images/Sc_Arrow_BelowValue_Def.png", qsTrId("ID_ADJUST_STATE_ISTOTHERM_MIN"), "../images/Sc_Arrow_AboveValue_Def.png")
         }
    }

    // Tilt states
    states: [
        State {
            name: "0"
            PropertyChanges { target: scaleAndIsothermItem; rotation: 0 }
            PropertyChanges { target: scaleAndIsothermItem; height: grid.height - grid.topMargin - grid.bottomMargin - 2 * grid.verticalSpacing }
            PropertyChanges { target: scaleAndIsothermItem; x: grid.width - grid.rightMargin - grid.irRightMargin - grid.horizontalSpacing - width }
            PropertyChanges { target: scaleAndIsothermItem; y: grid.topMargin + grid.verticalSpacing }
        },
        State {
            name: "90"
            PropertyChanges { target: scaleAndIsothermItem; rotation: 90 }
            PropertyChanges { target: scaleAndIsothermItem; height: grid.width - grid.leftMargin - grid.irLeftMargin - grid.rightMargin - grid.irRightMargin - 2 * grid.horizontalSpacing }
            PropertyChanges { target: scaleAndIsothermItem; x: height - width + grid.leftMargin + grid.irLeftMargin + grid.horizontalSpacing }
            PropertyChanges { target: scaleAndIsothermItem; y: grid.height - grid.bottomMargin - grid.verticalSpacing }
        },
        State {
            name: "180"
            PropertyChanges { target: scaleAndIsothermItem; rotation: 180 }
            PropertyChanges { target: scaleAndIsothermItem; height: grid.height - grid.topMargin - grid.bottomMargin - 2 * grid.verticalSpacing }
            PropertyChanges { target: scaleAndIsothermItem; x: grid.leftMargin + grid.irLeftMargin + grid.horizontalSpacing - width }
            PropertyChanges { target: scaleAndIsothermItem; y: grid.height - grid.verticalSpacing - grid.topMargin }
        },
        State {
            name: "270"
            PropertyChanges { target: scaleAndIsothermItem; rotation: 270 }
            PropertyChanges { target: scaleAndIsothermItem; height: grid.width - grid.leftMargin - grid.irLeftMargin - grid.rightMargin - grid.irRightMargin - 2 * grid.horizontalSpacing }
            PropertyChanges { target: scaleAndIsothermItem; x: grid.leftMargin + grid.irLeftMargin + grid.horizontalSpacing - width }
            PropertyChanges { target: scaleAndIsothermItem; y: grid.topMargin + grid.verticalSpacing }
        }
    ]


    Image {
        id: laserOnIndicatorPlaceholder
        source: "../images/Sc_Status_Laser.png"
        anchors.top: parent.top
        visible: false

        x: {
                updateLaserPosDelay.restart()

                if (greenbox.appState !== GreenBox.FacetLiveView && greenbox.appState !== GreenBox.FacetMediaControlState)
                    return grid.width - laserOnIndicatorPlaceholder.sourceSize.width - grid.horizontalSpacing;
                else if(scaleItem.visible)
                    return upperRow.x - laserOnIndicatorPlaceholder.sourceSize.width - grid.horizontalSpacing
                else
                    return parent.width - laserOnIndicatorPlaceholder.sourceSize.width
        }

        Timer {
            id: updateLaserPosDelay
            interval: 50
            repeat: false
            onTriggered: laserOnIndicatorPlaceholder.updateLaserPlaceHolder()

        }

        function updateLaserPlaceHolder()
        {
            var laserIndicatorElement = greenbox.GetNamedObject("laserOnIndicator")
            var coordReference = (greenbox.appState === GreenBox.FacetLiveView
                    || greenbox.appState === GreenBox.FacetMediaControlState) ?  scaleAndIsothermItem : greenbox.GetNamedObject("root")

            laserIndicatorElement.updateLocation(coordReference, laserOnIndicatorPlaceholder)
         }

    }
}
