import QtQuick 1.1

// ThumbnailArchive
// ----------------
// The thumbnail archive.

GridView {

    // Signals and properties

    signal hideArchive
    signal showPreviewArchive

    // Debug output

//    onHideArchive: console.log("ThumbnailArchive::hideArchive()")
//    onShowPreviewArchive: console.log("ThumbnailArchive::showPreviewArchive()")

    // Property values
    property bool isScrolling: false

    id: thumbnailArchive
    model: archiveHandler.model
    anchors.fill: parent
    anchors.leftMargin: grid.leftMargin + grid.irLeftMargin
    anchors.rightMargin: grid.rightMargin
    anchors.topMargin: grid.topMargin
    anchors.bottomMargin: grid.bottomMargin
    cellWidth: grid.horizontalSpacing + grid.archiveCellWidth
    cellHeight: grid.verticalSpacing + grid.archiveCellHeight
    snapMode: GridView.SnapToRow

    delegate: ThumbnailDelegate { }

    highlight: Item {
        Rectangle {
            x: grid.horizontalSpacing
            y: grid.verticalSpacing
            width: grid.archiveCellWidth
            height: grid.archiveCellHeight
            color: colors.darkGrey
        }
    }
    highlightFollowsCurrentItem: true
    highlightMoveDuration: 1

    onCurrentIndexChanged: archiveHandler.model.currentIndex = currentIndex

    Behavior on opacity {
        NumberAnimation {
            duration: 300
            easing.type: Easing.OutQuad
        }
    }

    // Used for setting to last item when pressing down at the end. Kind of a HACK!
    property int indexMemory: -1

    Keys.onPressed: {
        // Special case when we've reached the bottom.
        var lastIndex = thumbnailArchive.count -1
        if (event.key === keys.down && currentIndex < lastIndex) {
            indexMemory = currentIndex
        }
    }

    // Keyboard handling
    Keys.onReleased: {
        if (event.key === keys.back) {
            hideArchive()
            event.accepted = true
        }
        else if (event.key === keys.select) {
            showPreviewArchive()
            event.accepted = true
        }
        else if ((event.key === keys.up || event.key === keys.down) && event.isAutoRepeat && !isScrolling)
        {
            isScrolling = true
        }
        else if ((event.key === keys.up || event.key === keys.down) && !event.isAutoRepeat && isScrolling)
        {
            positionViewAtIndex(currentIndex, ListView.Contain)
            isScrolling = false
        }

        var lastIndex = thumbnailArchive.count -1
        if (indexMemory === currentIndex && event.key === keys.down && currentIndex < lastIndex) {
            currentIndex = lastIndex
            positionViewAtIndex(currentIndex, ListView.Contain)
        }
        indexMemory = -1
    }
}
