import QtQuick 1.1

/**
 *  This QML file is the delegate for the C++ class TrippleListMenu.
 */

FocusScope {
    width: grid.cellWidth * 5
    height: 153
    id: root
    property QtObject model: null

    function close() { /* Needed for consistency */  }

    BorderImage {
        anchors.fill: parent
        border { left: 10; top: 10; right: 10; bottom: 10 }
        source: "../images/Bg_MenuMiniWidget_Def.png"

        // Header
        Text {
            id: headerText
            anchors.top: parent.top
            anchors.topMargin: 3*grid.verticalSpacing
            anchors.left: parent.left
            anchors.leftMargin: 4*grid.horizontalSpacing
            font.pixelSize: fonts.smallSize
            font.family: fonts.family
            color: colors.white
            text: root.model === null ? "" : qsTrId(root.model.header) + translation.update
        }

        // Blue line
        BorderImage {
            id: toplineImage
            source: "../images/Bg_DialoguePopUp_Line_Def.png"
            border { left: 50; top: 0; right: 50; bottom: 0 }
            anchors.left: parent.left
            anchors.leftMargin: grid.horizontalSpacing
            anchors.right: parent.right
            anchors.rightMargin: grid.horizontalSpacing
            anchors.top: headerText.bottom
            anchors.topMargin: 3*grid.verticalSpacing
        }

        // Row of lists
        Row {
            NewValueSelectorList {
                id: firstList
                model: root.model == null ? null : root.model.firstModel
                onActivated: root.model.setSelection(firstList.currentIndex, secondList.currentIndex, thirdList.currentIndex);
                width: root.width / 3
                height: root.height
                listViewHeight: root.height
                focus: true
                Keys.onRightPressed: secondList.focus = true
                showScrollArrows: true
                visibleRows: 1
                currentIndex: root.model === null ? 0 : root.model.firstSelectedIndex

                onActiveFocusChanged: {
                    if (focus)
                    {
                        setFullHeight(false)
                        showScrollArrows = true
                        footerText.text = root.model.firstFooterArg === "" ? qsTrId(root.model.firstFooter) : qsTrId(root.model.firstFooter).arg(qsTrId(root.model.firstFooterArg))
                    }
                    else
                    {
                        setFullHeight(false)
                        showScrollArrows = false
                    }
                }
            }

            NewValueSelectorList {
                id: secondList
                model: root.model == null ? null : root.model.secondModel
                onActivated: root.model.setSelection(firstList.currentIndex, secondList.currentIndex, thirdList.currentIndex);
                width: root.width / 3
                height: root.height
                listViewHeight: grid.cellHeight
                Keys.onLeftPressed: firstList.focus = true
                Keys.onRightPressed: thirdList.focus = true
                showScrollArrows: false
                visibleRows: 1
                currentIndex: root.model === null ? 0 : root.model.secondSelectedIndex

                onActiveFocusChanged: {
                    if (focus)
                    {
                        setFullHeight(false)
                        showScrollArrows = true
                        footerText.text = root.model.secondFooterArg === "" ? qsTrId(root.model.secondFooter) : qsTrId(root.model.secondFooter).arg(qsTrId(root.model.secondFooterArg))
                    }
                    else
                    {
                        setFullHeight(false)
                        showScrollArrows = false
                    }
                }
            }

            NewValueSelectorList {
                id: thirdList
                model: root.model == null ? null : root.model.thirdModel
                onActivated: root.model.setSelection(firstList.currentIndex, secondList.currentIndex, thirdList.currentIndex);
                width: root.width / 3
                height: root.height
                listViewHeight: grid.cellHeight
                Keys.onLeftPressed: secondList.focus = true
                showScrollArrows: false
                visibleRows: 1
                currentIndex: root.model === null ? 0 : root.model.thirdSelectedIndex

                onActiveFocusChanged: {
                    if (focus)
                    {
                        setFullHeight(false)
                        showScrollArrows = true
                        footerText.text = root.model.thirdFooterArg === "" ? qsTrId(root.model.thirdFooter) : qsTrId(root.model.thirdFooter).arg(qsTrId(root.model.thirdFooterArg))
                    }
                    else
                    {
                        setFullHeight(false)
                        showScrollArrows = false
                    }
                }
            }
        }

        // Blue line
        BorderImage {
            id: bottomlineImage
            source: "../images/Bg_DialoguePopUp_Line_Def.png"
            border { left: 50; top: 0; right: 50; bottom: 0 }
            anchors.left: parent.left
            anchors.leftMargin: grid.horizontalSpacing
            anchors.right: parent.right
            anchors.rightMargin: grid.horizontalSpacing
            anchors.bottom: footerText.top
            anchors.bottomMargin: 2*grid.verticalSpacing
        }

        // Footer
        Text {
            id: footerText
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 3*grid.verticalSpacing
            anchors.left: parent.left
            anchors.leftMargin: 4*grid.horizontalSpacing
            font.pixelSize: fonts.smallSize
            font.family: fonts.family
            color: colors.white
            text: ""
        }
    }

    Keys.onReleased: {
        event.accepted = false;
        if (event.key === keys.back) {
            root.visible =  false
            event.accepted = true
        }
    }

    Keys.onUpPressed: ; // eat those events
    Keys.onDownPressed: ; // eat those events
    Keys.onLeftPressed: ; // eat those events
    Keys.onRightPressed: ; // eat those events
}
