import QtQuick 1.1

Item {
    property Item previous: null
    id: root
    // Boring stuff to work with the settings framework.
    property string name: "unused"
    property alias basePage: settingsHeader.basePage
    property variant model: null
    function runShowParentAnimation() { x = grid.width }
    function runHideParentAnimation() { x = 0 }
    // end boring

    Behavior on x { NumberAnimation { duration: 200; easing.type: Easing.InOutQuad } }

    signal hideSignal

    x: grid.width
    y: 0
    width: grid.width
    height: grid.height

    // Animations used to slide pages in and out
    ParallelAnimation {
        id: showAnimation
        NumberAnimation {
            target: page; property: "x"; to: previous === null ? 0 : grid.width;
            duration: previous !== null ? 200 : 0; easing.type: Easing.InOutQuad
        }
        ScriptAction {
            script: {
                if (previous !== null) {
                    previous.runHideParentAnimation()
                    previous.model.isNowActive(false)
                }
            }
        }
    }

    SequentialAnimation {
        id: hideAnimation
        ParallelAnimation {
            NumberAnimation {
                target: page; property: "x"; to: grid.width;
                duration: previous !== null ? 200 : 0; easing.type: Easing.InOutQuad
            }
            ScriptAction { script: { if (previous !== null) { previous.runShowParentAnimation();} } }
        }
        ScriptAction { script: { focus = false; if (previous !== null) {previous.forceActiveFocus(); } hideSignal() } }
    }

    // Functions used to show and hide pages
    function show () {
        showAnimation.start()
        settingsView.forceActiveFocus()
        wifiNetworksModel.isNowActive(true)
    }

    function hide () {
        hideAnimation.start()
        wifiNetworksModel.isNowActive(false)
    }

    Keys.onPressed: {
        if (event.key === keys.back || event.key == keys.left) {
            event.accepted = true
        }
    }
    Keys.onReleased: {
        if (event.key === keys.back || event.key == keys.left) {
            hide()
            event.accepted = true
        }
    }

    Rectangle {
        color: "black"
        width: grid.width
        height: grid.height

        SettingsHeader {
            id: settingsHeader
            setting: qsTrId(wifiNetworksModel.header)
            onClicked: hide()
        }

        ListView {
            focus: true
            id: settingsView
            model: wifiNetworksModel.networks

            anchors.top: settingsHeader.bottom
            width: parent.width
            anchors.bottom: settingsFooter.top
            clip: true  // Avoids items outside listview

            boundsBehavior: Flickable.StopAtBounds
            highlightMoveDuration: 5
            preferredHighlightBegin: grid.cellHeight + 2 * grid.verticalSpacing
            preferredHighlightEnd: height - grid.cellHeight - 2 * grid.verticalSpacing
            highlightRangeMode: ListView.ApplyRange

            delegate:
                Item {
                    focus: true
                    width: settingsView.width
                    height: grid.cellHeight + grid.verticalSpacing
                    property bool isCurrentItem: index == settingsView.currentIndex

                    // Highlight
                    BorderImage {
                        visible: isCurrentItem
                        source: "../images/Bg_FullScreen_Marker_Foc.png"
                        border.left: 5; border.top: 5; border.right: 5; border.bottom: 5
                        width: grid.width - 4 * grid.horizontalSpacing
                        height: grid.cellHeight - 2 * grid.verticalSpacing
                        anchors.centerIn: parent
                    }

                    Text {
                        id: settingTextItem
                        anchors.left: parent.left
                        anchors.leftMargin: grid.unit*5
                        width: grid.width - icons.width - x
                        anchors.verticalCenter: parent.verticalCenter
                        color: isCurrentItem ? colors.textFocused : colors.textNormal
                        font.pixelSize: fonts.smallSize
                        font.family: fonts.family
                        text: qsTrId(modelData.title)
                        clip: isCurrentItem
                        elide: isCurrentItem ? Text.ElideRight : Text.ElideNone
                    }
                    Item {
                        visible: !isCurrentItem
                        anchors.right: parent.right
                        height: settingTextItem.height
                        y: settingTextItem.y
                        width: icons.width + 20
                        Rectangle {
                            width: parent.height
                            height: parent.width
                            x: height
                            transformOrigin: Item.TopLeft
                            rotation: 90
                            gradient: Gradient {
                                GradientStop { position: 0.4; color: "#a0000000" }
                                GradientStop { position: 1; color: "#00000000" } // pure transparant
                            }
                        }
                    }

                    Row {
                        id: icons
                        visible: modelData.objectName == ""
                        anchors { right: parent.right; top: parent.top; bottom: parent.bottom }
                        spacing: grid.horizontalSpacing

                        Image { // wifi-strength / encryption indicators
                            id: strengthIcon
                            anchors.verticalCenter: parent.verticalCenter
                            source: !parent.visible ? "" : ("../images/Sc_Status_" +
                                    (modelData.isEncrypted ?
                                      (modelData.signalStrength == 0 ? "Locked" : "Wifi_Locked_0" + modelData.signalStrength)
                                      : ("Wifi0" + modelData.signalStrength))
                                    + "_Def.png")
                        }
                        Item {
                            id: spinner

                            visible: modelData.connecting
                            width: activeIcon.width
                            height: width
                            anchors.verticalCenter: parent.verticalCenter

                            Spinner {
                                anchors.centerIn: parent
                                width: settingTextItem.height
                            }
                        }
                        Image {
                            id: activeIcon
                            visible: ! modelData.connecting
                            anchors.verticalCenter: parent.verticalCenter
                            source: "../images/Ic_ListChoice_RadioButton" +
                                        (modelData.active ? "Checked" : "") +
                                        (isCurrentItem ? "_Sel.png" : "_Def.png")
                        }
                        Item { // spacer
                            height: 1; width: grid.horizontalSpacing * 3
                        }
                    }
                    Image {
                        visible: modelData.objectName == "next"
                        anchors.right: parent.right
                        anchors.rightMargin: grid.horizontalSpacing
                        anchors.verticalCenter: parent.verticalCenter
                        source: "../images/Ic_List_ArrowRight" + (isCurrentItem ? "_Sel.png" : "_Def.png")
                    }

                    function exec() {
                        // modelData does not seem to work correctly. See the
                        // usage of connectTo2 instead.
                        if (modelData.objectName == "next") {
                            pageLoader.source = "SettingsPage.qml";
                        } else {
                            //wifiNetworksModel.connectTo(modelData)
                            wifiNetworksModel.connectTo2(settingsView.currentIndex)
                        }
                    }

                    MouseArea {
                        anchors.fill: parent
                        onClicked: exec()
                    }

                    Keys.onReleased: {
                        if (event.key === keys.select || event.key === keys.right) {
                            onClicked: exec()
                            event.accepted = true
                        }
                    }
                }
        }

        // Top scroll fader
        Image {
            id: topScrollCover
            visible: settingsView.visibleArea.yPosition > 0 ? true : false
            source: "../images/Bg_Options_ScrollCoverTop.png"
            anchors.top: settingsView.top
            x: grid.unit*2
        }

        // Bottom scroll fader
        Image {
            id: bottomScrollCover
            visible: (settingsView.contentHeight > settingsView.height) &&
                     (settingsView.visibleArea.yPosition < 1 - settingsView.visibleArea.heightRatio - 0.01) ? true : false
            source: "../images/Bg_Options_ScrollCoverBottom.png"
            anchors.bottom: settingsView.bottom
            x: grid.unit*2
        }

        // Scroll bar
        Rectangle {
            id: scrollBar
            visible: settingsView.contentHeight > settingsView.height ? true : false
            anchors.right: parent.right
            anchors.top: settingsView.top
            anchors.bottom: settingsView.bottom
            width: 3
            color: "#323232"

            // Slider
            Rectangle {
                color: "#0066cc"
                anchors.right: parent.right
                width: parent.width
                height: settingsView.height * settingsView.visibleArea.heightRatio
                y: settingsView.visibleArea.yPosition * settingsView.height
            }
        }

        SettingsFooter {
            id: settingsFooter
            visible: false
            height: visible ? grid.cellHeight + grid.bottomMargin : 0
            anchors.left: parent.left
            anchors.bottom: parent.bottom
        }
    }

    // Loader used to load and focus next page
    Loader {
        id: pageLoader
        onLoaded: {
            console.log("SettingsPage " + item.name + " loaded")
            item.previous = root
            item.basePage = false
            item.model = greenbox.GetNamedObject("wifiAddNetworkSettingsModel")
            item.show()
        }
    }

    Connections {
        target: pageLoader.item
        onHideSignal: {
            pageLoader.source = ""
            wifiNetworksModel.isNowActive(true)
            settingsView.positionViewAtIndex(settingsView.currentIndex, ListView.Center)
            settingsView.forceActiveFocus()
        }
    }
}
